@@ -2,8 +2,8 @@
 
   bignum.c -
 
-  $Author: gotoyuzo $
-  $Date: 2004/11/16 19:43:24 $
+  $Author: nobu $
+  $Date: 2005/09/06 23:22:56 $
   created at: Fri Jun 10 00:48:55 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -42,11 +42,11 @@ static VALUE
 bignew_1(klass, len, sign)
     VALUE klass;
     long len;
-    char sign;
+    int sign;
 {
     NEWOBJ(big, struct RBignum);
     OBJSETUP(big, klass, T_BIGNUM);
-    big->sign = sign;
+    big->sign = (char)sign;
     big->len = len;
     big->digits = ALLOC_N(BDIGIT, len);
 
@@ -65,10 +65,10 @@ rb_big_clone(x)
     return z;
 }
 
+/* modify a bignum by 2's complement */
 static void
-get2comp(x, carry)		/* get 2's complement */
+get2comp(x)
     VALUE x;
-    int carry;
 {
     long i = RBIGNUM(x)->len;
     BDIGIT *ds = BDIGITS(x);
@@ -81,11 +81,10 @@ get2comp(x, carry)		/* get 2's complement */
 	ds[i++] = BIGLO(num);
 	num = BIGDN(num);
     } while (i < RBIGNUM(x)->len);
-    if (!carry) return;
-    if ((ds[RBIGNUM(x)->len-1] & (1<<(BITSPERDIG-1))) == 0) {
+    if (num != 0) {
 	REALLOC_N(RBIGNUM(x)->digits, BDIGIT, ++RBIGNUM(x)->len);
 	ds = BDIGITS(x);
-	ds[RBIGNUM(x)->len-1] = ~0;
+	ds[RBIGNUM(x)->len-1] = RBIGNUM(x)->sign ? ~0 : 1;
     }
 }
 
@@ -93,7 +92,7 @@ void
 rb_big_2comp(x)			/* get 2's complement */
     VALUE x;
 {
-    get2comp(x, Qtrue);
+    get2comp(x);
 }
 
 static VALUE
@@ -751,7 +750,7 @@ unsigned long
 rb_big2ulong_pack(x)   
     VALUE x;  
 {   
-    unsigned long num = big2ulong(x, "unsigned long", Qfalse);
+    unsigned long num = big2ulong(x, "unsigned long");
     if (!RBIGNUM(x)->sign) {
 	return -num;
     }
@@ -980,8 +979,8 @@ rb_big_eq(x, y)
 	    volatile double a, b;
 
 	    a = RFLOAT(y)->value;
+	    if (isnan(a)) return Qfalse;
 	    b = rb_big2dbl(x);
-	    if (isnan(a) || isnan(b)) return Qfalse;
 	    return (a == b)?Qtrue:Qfalse;
 	}
       default:
@@ -1050,13 +1049,15 @@ rb_big_neg(x)
     VALUE x;
 {
     VALUE z = rb_big_clone(x);
-    long i = RBIGNUM(x)->len;
-    BDIGIT *ds = BDIGITS(z);
+    long i;
+    BDIGIT *ds;
 
-    if (!RBIGNUM(x)->sign) get2comp(z, Qtrue);
+    if (!RBIGNUM(x)->sign) get2comp(z);
+    ds = BDIGITS(z);
+    i = RBIGNUM(x)->len;
     while (i--) ds[i] = ~ds[i];
-    if (RBIGNUM(x)->sign) get2comp(z, Qfalse);
     RBIGNUM(z)->sign = !RBIGNUM(z)->sign;
+    if (RBIGNUM(x)->sign) get2comp(z);
 
     return bignorm(z);
 }
@@ -1111,7 +1112,7 @@ bigsub(x, y)
 static VALUE
 bigadd(x, y, sign)
     VALUE x, y;
-    char sign;
+    int sign;
 {
     VALUE z;
     BDIGIT_DBL num;
@@ -1648,11 +1649,11 @@ rb_big_and(xx, yy)
     }
     if (!RBIGNUM(y)->sign) {
 	y = rb_big_clone(y);
-	get2comp(y, Qtrue);
+	get2comp(y);
     }
     if (!RBIGNUM(x)->sign) {
 	x = rb_big_clone(x);
-	get2comp(x, Qtrue);
+	get2comp(x);
     }
     if (RBIGNUM(x)->len > RBIGNUM(y)->len) {
 	l1 = RBIGNUM(y)->len;
@@ -1677,7 +1678,7 @@ rb_big_and(xx, yy)
     for (; i<l2; i++) {
 	zds[i] = sign?0:ds2[i];
     }
-    if (!RBIGNUM(z)->sign) get2comp(z, Qfalse);
+    if (!RBIGNUM(z)->sign) get2comp(z);
     return bignorm(z);
 }
 
@@ -1702,14 +1703,13 @@ rb_big_or(xx, yy)
     if (FIXNUM_P(y)) {
 	y = rb_int2big(FIX2LONG(y));
     }
-
     if (!RBIGNUM(y)->sign) {
 	y = rb_big_clone(y);
-	get2comp(y, Qtrue);
+	get2comp(y);
     }
     if (!RBIGNUM(x)->sign) {
 	x = rb_big_clone(x);
-	get2comp(x, Qtrue);
+	get2comp(x);
     }
     if (RBIGNUM(x)->len > RBIGNUM(y)->len) {
 	l1 = RBIGNUM(y)->len;
@@ -1734,7 +1734,7 @@ rb_big_or(xx, yy)
     for (; i<l2; i++) {
 	zds[i] = sign?ds2[i]:(BIGRAD-1);
     }
-    if (!RBIGNUM(z)->sign) get2comp(z, Qfalse);
+    if (!RBIGNUM(z)->sign) get2comp(z);
 
     return bignorm(z);
 }
@@ -1761,14 +1761,13 @@ rb_big_xor(xx, yy)
     if (FIXNUM_P(y)) {
 	y = rb_int2big(FIX2LONG(y));
     }
-
     if (!RBIGNUM(y)->sign) {
 	y = rb_big_clone(y);
-	get2comp(y, Qtrue);
+	get2comp(y);
     }
     if (!RBIGNUM(x)->sign) {
 	x = rb_big_clone(x);
-	get2comp(x, Qtrue);
+	get2comp(x);
     }
     if (RBIGNUM(x)->len > RBIGNUM(y)->len) {
 	l1 = RBIGNUM(y)->len;
@@ -1795,7 +1794,7 @@ rb_big_xor(xx, yy)
     for (; i<l2; i++) {
 	zds[i] = sign?ds2[i]:~ds2[i];
     }
-    if (!RBIGNUM(z)->sign) get2comp(z, Qfalse);
+    if (!RBIGNUM(z)->sign) get2comp(z);
 
     return bignorm(z);
 }
@@ -1867,7 +1866,7 @@ rb_big_rshift(x, y)
     }
     if (!RBIGNUM(x)->sign) {
 	x = rb_big_clone(x);
-	get2comp(x, Qtrue);
+	get2comp(x);
     }
     xds = BDIGITS(x);
     i = RBIGNUM(x)->len; j = i - s1;
@@ -1882,7 +1881,7 @@ rb_big_rshift(x, y)
 	num = BIGUP(xds[i]);
     }
     if (!RBIGNUM(x)->sign) {
-	get2comp(z, Qfalse);
+	get2comp(z);
     }
     return bignorm(z);
 }
@@ -1927,7 +1926,7 @@ rb_big_aref(x, y)
     if (!RBIGNUM(x)->sign) {
 	if (s1 >= RBIGNUM(x)->len) return INT2FIX(1);
 	x = rb_big_clone(x);
-	get2comp(x, Qtrue);
+	get2comp(x);
     }
     else {
 	if (s1 >= RBIGNUM(x)->len) return INT2FIX(0);
@@ -1970,8 +1969,11 @@ rb_big_coerce(x, y)
     if (FIXNUM_P(y)) {
 	return rb_assoc_new(rb_int2big(FIX2LONG(y)), x);
     }
+    else if (TYPE(y) == T_BIGNUM) {
+       return rb_assoc_new(y, x);
+    }
     else {
-	rb_raise(rb_eTypeError, "Can't coerce %s to Bignum",
+	rb_raise(rb_eTypeError, "can't coerce %s to Bignum",
 		 rb_obj_classname(y));
     }
     /* not reached */
@@ -2010,11 +2012,13 @@ rb_big_rand(max, rand_buf)
 	return rb_float_new(rand_buf[0]);
     }
     v = bignew(len,1);
+    len--;
+    BDIGITS(v)[len] = BDIGITS(max)[len] * rand_buf[len];    
     while (len--) {
 	BDIGITS(v)[len] = ((BDIGIT)~0) * rand_buf[len];
     }
 
-    return rb_big_modulo((VALUE)v, max);
+    return v;
 }
 
 /*
