@@ -1,19 +1,13 @@
 #! /usr/local/bin/ruby
 # -*- ruby -*-
 
-$extension = nil
-$extstatic = nil
 $force_static = nil
 $install = nil
 $destdir = nil
-$dryrun = false
 $clean = nil
 $nodynamic = nil
 $extinit = nil
 $extobjs = nil
-$extflags = ""
-$extlibs = nil
-$extpath = nil
 $ignore = nil
 $message = nil
 
@@ -22,6 +16,7 @@ alias $PROGRAM_NAME $0
 alias $0 $progname
 
 $extlist = []
+$extupdate = false
 $compiled = {}
 
 $:.replace([Dir.pwd])
@@ -75,18 +70,14 @@ def extract_makefile(makefile, keep = true)
     return false
   end
   $target = target
-  $extconf_h = m[/^RUBY_EXTCONF_H[ \t]*=[ \t]*(\S+)/, 1]
-  $static ||= m[/^EXTSTATIC[ \t]*=[ \t]*(\S+)/, 1] || false
   /^STATIC_LIB[ \t]*=[ \t]*\S+/ =~ m or $static = nil
   $preload = Shellwords.shellwords(m[/^preload[ \t]*=[ \t]*(.*)/, 1] || "")
-  $DLDFLAGS += " " + (m[/^dldflags[ \t]*=[ \t]*(.*)/, 1] || "")
+  $DLDFLAGS += " " + (m[/^DLDFLAGS[ \t]*=[ \t]*(.*)/, 1] || "")
   if s = m[/^LIBS[ \t]*=[ \t]*(.*)/, 1]
     s.sub!(/^#{Regexp.quote($LIBRUBYARG)} */, "")
     s.sub!(/ *#{Regexp.quote($LIBS)}$/, "")
     $libs = s
   end
-  $objs = (m[/^OBJS[ \t]*=[ \t](.*)/, 1] || "").split
-  $srcs = (m[/^SRCS[ \t]*=[ \t](.*)/, 1] || "").split
   $LOCAL_LIBS = m[/^LOCAL_LIBS[ \t]*=[ \t]*(.*)/, 1] || ""
   $LIBPATH = Shellwords.shellwords(m[/^libpath[ \t]*=[ \t]*(.*)/, 1] || "") - %w[$(libdir) $(topdir)]
   true
@@ -112,51 +103,29 @@ def extmake(target)
     Dir.chdir target
     top_srcdir = $top_srcdir
     topdir = $topdir
-    hdrdir = $hdrdir
+    mk_srcdir = CONFIG["srcdir"]
+    mk_topdir = CONFIG["topdir"]
     prefix = "../" * (target.count("/")+1)
-    $top_srcdir = relative_from(top_srcdir, prefix)
-    $hdrdir = relative_from(hdrdir, prefix)
+    $hdrdir = $top_srcdir = relative_from(top_srcdir, prefix)
     $topdir = prefix + $topdir
     $target = target
     $mdir = target
     $srcdir = File.join($top_srcdir, "ext", $mdir)
     $preload = nil
-    $objs = ""
-    $srcs = ""
     $compiled[target] = false
     makefile = "./Makefile"
     ok = File.exist?(makefile)
     unless $ignore
-      rbconfig0 = Config::CONFIG
-      mkconfig0 = CONFIG
-      rbconfig = {
-	"hdrdir" => $hdrdir,
-	"srcdir" => $srcdir,
-	"topdir" => $topdir,
-      }
-      mkconfig = {
-	"hdrdir" => "$(top_srcdir)",
-	"srcdir" => "$(top_srcdir)/ext/#{$mdir}",
-	"topdir" => $topdir,
-      }
-      rbconfig0.each_pair {|key, val| rbconfig[key] ||= val.dup}
-      mkconfig0.each_pair {|key, val| mkconfig[key] ||= val.dup}
-      Config.module_eval {
-	remove_const(:CONFIG)
-	const_set(:CONFIG, rbconfig)
-	remove_const(:MAKEFILE_CONFIG)
-	const_set(:MAKEFILE_CONFIG, mkconfig)
-      }
-      Object.class_eval {
-	remove_const(:CONFIG)
-	const_set(:CONFIG, mkconfig)
-      }
+      Config::CONFIG["hdrdir"] = $hdrdir
+      Config::CONFIG["srcdir"] = $srcdir
+      Config::CONFIG["topdir"] = $topdir
+      CONFIG["hdrdir"] = ($hdrdir == top_srcdir) ? top_srcdir : "$(topdir)"+top_srcdir[2..-1]
+      CONFIG["srcdir"] = "$(hdrdir)/ext/#{$mdir}"
+      CONFIG["topdir"] = $topdir
       begin
-	$extconf_h = nil
-	ok &&= extract_makefile(makefile)
-	if (($extconf_h && !File.exist?($extconf_h)) ||
+	if (!(ok &&= extract_makefile(makefile)) ||
 	    !(t = modified?(makefile, MTIMES)) ||
-	    ["#{$srcdir}/makefile.rb", "#{$srcdir}/extconf.rb", "#{$srcdir}/depend"].any? {|f| modified?(f, [t])})
+            %W"#{$srcdir}/makefile.rb #{$srcdir}/extconf.rb #{$srcdir}/depend".any? {|f| modified?(f, [t])})
         then
 	  ok = false
           init_mkmf
@@ -169,7 +138,7 @@ def extmake(target)
 	  else
 	    create_makefile(target)
 	  end
-	  $defs << "-DRUBY_EXPORT" if $static
+	  $extupdate = true
 	  ok = File.exist?(makefile)
 	end
       rescue SystemExit
@@ -199,11 +168,7 @@ def extmake(target)
       $ignore or $continue or return false
     end
     $compiled[target] = true
-    if $clean
-      FileUtils.rm_f("mkmf.log")
-      if $clean != true
-	FileUtils.rm_f([makefile, $extconf_h || "extconf.h"])
-      end
+    if $clean and $clean != true
       File.unlink(makefile) rescue nil
     end
     if $static
@@ -217,21 +182,12 @@ def extmake(target)
       $extpath |= $LIBPATH
     end
   ensure
-    unless $ignore
-      Config.module_eval {
-	remove_const(:CONFIG)
-	const_set(:CONFIG, rbconfig0)
-	remove_const(:MAKEFILE_CONFIG)
-	const_set(:MAKEFILE_CONFIG, mkconfig0)
-      }
-      Object.class_eval {
-	remove_const(:CONFIG)
-	const_set(:CONFIG, mkconfig0)
-      }
-    end
-    $top_srcdir = top_srcdir
+    $hdrdir = $top_srcdir = top_srcdir
+    Config::CONFIG["topdir"] = topdir
+    CONFIG["srcdir"] = mk_srcdir
+    CONFIG["topdir"] = mk_topdir
+    CONFIG.delete("hdrdir")
     $topdir = topdir
-    $hdrdir = hdrdir
     Dir.chdir dir
   end
   begin
@@ -260,7 +216,8 @@ def parse_args()
       if ($extstatic = v) == false
         $extstatic = []
       elsif v
-        $force_static = true if $extstatic.delete("static")
+        $force_static = true
+        $extstatic.delete("static")
         $extstatic = nil if $extstatic.empty?
       end
     end
@@ -274,7 +231,6 @@ def parse_args()
       $make = v || 'make'
     end
     opts.on('--make-flags=FLAGS', '--mflags', Shellwords) do |v|
-      v.grep(/\A([-\w]+)=(.*)/) {$configure_args["--#{$1}"] = $2}
       if arg = v.first
         arg.insert(0, '-') if /\A[^-][^=]*\Z/ =~ arg
       end
@@ -298,7 +254,7 @@ def parse_args()
   $mflags.unshift(*rest) unless rest.empty?
 
   def $mflags.set?(flag)
-    grep(/\A-(?!-).*#{flag.chr}/i) { return true }
+    grep(/\A-(?!-).*#{'%c' % flag}/i) { return true }
     false
   end
   def $mflags.defined?(var)
@@ -315,7 +271,6 @@ def parse_args()
   $continue = $mflags.set?(?k)
   if $extout
     $extout = '$(topdir)/'+$extout
-    Config::CONFIG["extout"] = CONFIG["extout"] = $extout
     $extout_prefix = $extout ? "$(extout)$(target_prefix)/" : ""
     $mflags << "extout=#$extout" << "extout_prefix=#$extout_prefix"
   end
@@ -325,9 +280,9 @@ parse_args()
 
 if target = ARGV.shift and /^[a-z-]+$/ =~ target
   $mflags.push(target)
+  target = target.sub(/^(dist|real)(?=(?:clean)?$)/, '')
   case target
-  when /^(dist|real)?(clean)$/
-    target = $2
+  when /clean/
     $ignore ||= true
     $clean = $1 ? $1[0] : true
   when /^install\b/
@@ -349,21 +304,13 @@ end
 EXEEXT = CONFIG['EXEEXT']
 if CROSS_COMPILING
   $ruby = CONFIG['MINIRUBY']
-elsif sep = config_string('BUILD_FILE_SEPARATOR')
-  $ruby = "$(topdir:/=#{sep})#{sep}miniruby" + EXEEXT
+elsif $nmake
+  $ruby = '$(topdir:/=\\)\\miniruby' + EXEEXT
 else
   $ruby = '$(topdir)/miniruby' + EXEEXT
 end
-$ruby << " -I'$(topdir)'"
-unless CROSS_COMPILING
-  $ruby << " -I'$(top_srcdir)/lib'"
-  $ruby << " -I'$(extout)/$(arch)' -I'$(extout)/common'" if $extout
-  $ruby << " -I./- -I'$(top_srcdir)/ext' -rpurelib.rb"
-  ENV["RUBYLIB"] = "-"
-  ENV["RUBYOPT"] = "-rpurelib.rb"
-end
+$ruby << " -I'$(topdir)' -I'$(hdrdir)/lib'"
 $config_h = '$(topdir)/config.h'
-$mflags << "ruby=#$ruby"
 
 MTIMES = [__FILE__, 'rbconfig.rb', srcdir+'/lib/mkmf.rb'].collect {|f| File.mtime(f)}
 
@@ -378,7 +325,7 @@ end
 for dir in ["ext", File::join($top_srcdir, "ext")]
   setup = File::join(dir, CONFIG['setup'])
   if File.file? setup
-    f = open(setup)
+    f = open(setup) 
     while line = f.gets()
       line.chomp!
       line.sub!(/#.*$/, '')
@@ -403,33 +350,20 @@ end unless $extstatic
 
 ext_prefix = "#{$top_srcdir}/ext"
 exts = $static_ext.sort_by {|t, i| i}.collect {|t, i| t}
-if $extension
-  exts |= $extension.select {|d| File.directory?("#{ext_prefix}/#{d}")}
-else
-  withes, withouts = %w[--with --without].collect {|w|
-    if not (w = %w[-extensions -ext].collect {|opt|arg_config(w+opt)}).any?
-      proc {false}
-    elsif (w = w.grep(String)).empty?
-      proc {true}
-    else
-      proc {|c1| w.collect {|opt| opt.split(/,/)}.flatten.any?(&c1)}
-    end
-  }
-  cond = proc {|ext|
-    cond1 = proc {|n| File.fnmatch(n, ext, File::FNM_PATHNAME)}
-    withes.call(cond1) or !withouts.call(cond1)
-  }
-  exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
-    d = File.dirname(d)
-    d.slice!(0, ext_prefix.length + 1)
-    d
-  }.find_all {|ext|
-    with_config(ext, &cond)
-  }.sort
-end
+exts |= $extension if $extension
+exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
+  d = File.dirname(d)
+  d.slice!(0, ext_prefix.length + 1)
+  d
+}.sort unless $extension
 
 if $extout
-  extout = Config.expand("#{$extout}", Config::CONFIG.merge("topdir"=>$topdir))
+  Config.expand(extout = "#$extout", Config::CONFIG.merge("topdir"=>$topdir))
+  if $install
+    Config.expand(dest = "#{$destdir}#{$rubylibdir}")
+    FileUtils.cp_r(extout+"/.", dest, :verbose => true, :noop => $dryrun)
+    exit
+  end
   unless $ignore
     FileUtils.mkpath(extout)
   end
@@ -439,46 +373,26 @@ dir = Dir.pwd
 FileUtils::makedirs('ext')
 Dir::chdir('ext')
 
-hdrdir = $hdrdir
 $hdrdir = $top_srcdir = relative_from(srcdir, $topdir = "..")
 exts.each do |d|
   extmake(d) or abort
 end
-$top_srcdir = srcdir
+$hdrdir = $top_srcdir = srcdir
 $topdir = "."
-$hdrdir = hdrdir
 
-extinit = Struct.new(:c, :o) {
-  def initialize(src)
-    super("#{src}.c", "#{src}.#{$OBJEXT}")
-  end
-}.new("extinit")
 if $ignore
-  FileUtils.rm_f(extinit.to_a) if $clean
+  FileUtils.rm_f(%W"extinit.c extinit.#{$OBJEXT}") if $clean
   Dir.chdir ".."
   if $clean
     Dir.rmdir('ext') rescue nil
-    if $extout
-      FileUtils.rm_rf([extout+"/common", extout+"/include/ruby", extout+"/rdoc"])
-      FileUtils.rm_rf(extout+"/"+CONFIG["arch"])
-      if $clean != true
-	FileUtils.rm_rf(extout+"/include/"+CONFIG["arch"])
-	FileUtils.rm_f($mflags.defined?("INSTALLED_LIST")||ENV["INSTALLED_LIST"]||".installed.list")
-	Dir.rmdir(extout+"/include") rescue nil
-	Dir.rmdir(extout) rescue nil
-      end
-    end
+    FileUtils.rm_rf(extout) if $extout
   end
   exit
 end
 
-$extinit ||= ""
-$extobjs ||= ""
-$extpath ||= []
-$extflags ||= ""
-$extlibs ||= []
-unless $extlist.empty?
-  $extinit << "\n" unless $extinit.empty?
+if $extlist.size > 0
+  $extinit ||= ""
+  $extobjs ||= ""
   list = $extlist.dup
   built = []
   while e = list.shift
@@ -492,33 +406,28 @@ unless $extlist.empty?
     end
     f = format("%s/%s.%s", s, i, $LIBEXT)
     if File.exist?(f)
-      $extinit << "    init(Init_#{i}, \"#{t}.so\");\n"
-      $extobjs << "ext/#{f} "
+      $extinit += "\tinit(Init_#{i}, \"#{t}.so\");\n"
+      $extobjs += "ext/#{f} "
       built << t
     end
   end
 
-  src = %{\
-#include "ruby.h"
-
-#define init(func, name) {void func _((void)); ruby_init_ext(name, func);}
-
-void ruby_init_ext _((const char *name, void (*init)(void)));
-
-void Init_ext _((void))\n{\n#$extinit}
-}
-  if !modified?(extinit.c, MTIMES) || IO.read(extinit.c) != src
-    open(extinit.c, "w") {|f| f.print src}
+  src = <<SRC
+extern char *ruby_sourcefile, *rb_source_filename();
+#define init(func, name) (ruby_sourcefile = src = rb_source_filename(name), func(), rb_provide(src))
+void Init_ext() {\n\tchar* src;\n#$extinit}
+SRC
+  if !modified?("extinit.c", MTIMES) || IO.read("extinit.c") != src
+    open("extinit.c", "w") {|f| f.print src}
   end
 
-  $extobjs = "ext/#{extinit.o} #{$extobjs}"
+  $extobjs = "ext/extinit.#{$OBJEXT} " + $extobjs
   if RUBY_PLATFORM =~ /m68k-human|beos/
     $extflags.delete("-L/usr/local/lib")
   end
   $extpath.delete("$(topdir)")
   $extflags = libpathflag($extpath) << " " << $extflags.strip
   conf = [
-    ['LIBRUBY_SO_UPDATE', '$(LIBRUBY_EXTS)'],
     ['SETUP', $setup],
     [enable_config("shared", $enable_shared) ? 'DLDOBJS' : 'EXTOBJS', $extobjs],
     ['EXTLIBS', $extlibs.join(' ')], ['EXTLDFLAGS', $extflags]
@@ -528,15 +437,11 @@ void Init_ext _((void))\n{\n#$extinit}
   puts conf
   $stdout.flush
   $mflags.concat(conf)
-else
-  FileUtils.rm_f(extinit.to_a)
 end
 rubies = []
-%w[RUBY RUBYW STATIC_RUBY].each {|r|
-  n = r
+%w[RUBY RUBYW].each {|r|
   if r = arg_config("--"+r.downcase) || config_string(r+"_INSTALL_NAME")
-    rubies << Config.expand(r+=EXEEXT)
-    $mflags << "#{n}=#{r}"
+    rubies << r+EXEEXT
   end
 }
 
@@ -544,21 +449,15 @@ Dir.chdir ".."
 unless $destdir.to_s.empty?
   $mflags.defined?("DESTDIR") or $mflags << "DESTDIR=#{$destdir}"
 end
+if !$extlist.empty? and $extupdate
+  rm_f(Config::CONFIG["LIBRUBY_SO"])
+end
 puts "making #{rubies.join(', ')}"
 $stdout.flush
 $mflags.concat(rubies)
 
 if $nmake == ?b
-  unless (vars = $mflags.grep(/\A\w+=/n)).empty?
-    open(mkf = "libruby.mk", "wb") do |tmf|
-      tmf.puts("!include Makefile")
-      tmf.puts
-      tmf.puts(*vars.map {|v| v.sub(/=/, " = ")})
-      tmf.puts("PRE_LIBRUBY_UPDATE = del #{mkf}")
-    end
-    $mflags.unshift("-f#{mkf}")
-    vars.each {|flag| flag.sub!(/\A/, "-D")}
-  end
+  $mflags.collect {|flag| flag.sub!(/\A(?=\w+=)/, "-D")}
 end
 system($make, *sysquote($mflags)) or exit($?.exitstatus)
 
