@@ -2,8 +2,8 @@
 
   gdbm.c -
 
-  $Author: matz $
-  $Date: 2004/12/09 02:35:28 $
+  $Author: usa $
+  $Date: 2005/06/20 07:53:20 $
   modified at: Mon Jan 24 15:59:52 JST 1994
 
 ************************************************/
@@ -72,6 +72,21 @@ fgdbm_close(obj)
     return Qnil;
 }
 
+static VALUE
+fgdbm_closed(obj)
+    VALUE obj;
+{
+    struct dbmdata *dbmp;
+
+    Data_Get_Struct(obj, struct dbmdata, dbmp);
+    if (dbmp == 0)
+	return Qtrue;
+    if (dbmp->di_dbm == 0)
+	return Qtrue;
+
+    return Qfalse;
+}
+
 static VALUE fgdbm_s_alloc _((VALUE));
 
 static VALUE
@@ -941,6 +956,7 @@ Init_gdbm()
 
     rb_define_method(rb_cGDBM, "initialize", fgdbm_initialize, -1);
     rb_define_method(rb_cGDBM, "close", fgdbm_close, 0);
+    rb_define_method(rb_cGDBM, "closed?", fgdbm_closed, 0);
     rb_define_method(rb_cGDBM, "[]", fgdbm_aref, 1);
     rb_define_method(rb_cGDBM, "fetch", fgdbm_fetch_m, -1);
     rb_define_method(rb_cGDBM, "[]=", fgdbm_store, 2);
