@@ -1,5 +1,5 @@
 /*
- * $Id: ossl.c,v 1.11.2.3 2004/12/16 09:39:54 gotoyuzo Exp $
+ * $Id: ossl.c,v 1.11.2.4 2005/09/10 00:54:29 gotoyuzo Exp $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -278,7 +278,7 @@ ossl_raise(VALUE exc, const char *fmt, ...)
     va_list args;
     char buf[BUFSIZ];
     const char *msg;
-    long e = ERR_get_error();
+    long e = ERR_peek_last_error();
     int len = 0;
 
     if (fmt) {
@@ -291,15 +291,34 @@ ossl_raise(VALUE exc, const char *fmt, ...)
 	    msg = ERR_error_string(e, NULL);
 	else
 	    msg = ERR_reason_error_string(e);
-	ERR_clear_error();
 	fmt = len ? ": %s" : "%s";
 	len += snprintf(buf+len, BUFSIZ-len, fmt, msg);
     }
+    if (dOSSL == Qtrue){ /* show all errors on the stack */
+	while ((e = ERR_get_error()) != 0){
+	    rb_warn("error on stack: %s", ERR_error_string(e, NULL));
+	}
+    }
+    ERR_clear_error();
 
     if(len > BUFSIZ) len = strlen(buf);
     rb_exc_raise(rb_exc_new(exc, buf, len));
 }
 
+VALUE
+ossl_get_errors()
+{
+    VALUE ary;
+    long e;
+
+    ary = rb_ary_new();
+    while ((e = ERR_get_error()) != 0){
+        rb_ary_push(ary, rb_str_new2(ERR_error_string(e, NULL)));
+    }
+
+    return ary;
+}
+
 /*
  * Debug
  */
@@ -411,6 +430,7 @@ Init_openssl()
     dOSSL = Qfalse;
     rb_define_module_function(mOSSL, "debug", ossl_debug_get, 0);
     rb_define_module_function(mOSSL, "debug=", ossl_debug_set, 1);
+    rb_define_module_function(mOSSL, "errors", ossl_get_errors, 0);
 
     /*
      * Get ID of to_der
