@@ -2,8 +2,8 @@
 /*
  * rubyext.c
  *
- * $Author: ocean $
- * $Date: 2006/01/30 15:11:57 $
+ * $Author: why $
+ * $Date: 2005/09/17 17:17:06 $
  *
  * Copyright (C) 2003-2005 why the lucky stiff
  */
@@ -75,10 +75,7 @@ void rb_syck_err_handler _((SyckParser *, char *));
 SyckNode * rb_syck_bad_anchor_handler _((SyckParser *, char *));
 void rb_syck_output_handler _((SyckEmitter *, char *, long));
 void rb_syck_emitter_handler _((SyckEmitter *, st_data_t));
-int syck_parser_assign_io _((SyckParser *, VALUE *));
-VALUE syck_scalar_alloc _((VALUE class));
-VALUE syck_seq_alloc _((VALUE class));
-VALUE syck_map_alloc _((VALUE class));
+int syck_parser_assign_io _((SyckParser *, VALUE));
 
 struct parser_xtra {
     VALUE data;  /* Borrowed this idea from marshal.c to fix [ruby-core:8067] problem */
@@ -107,7 +104,7 @@ rb_syck_compile(self, port)
     bytestring_t *sav; 
 
     SyckParser *parser = syck_new_parser();
-    taint = syck_parser_assign_io(parser, &port);
+    taint = syck_parser_assign_io(parser, port);
     syck_parser_handler( parser, syck_yaml2byte_handler );
     syck_parser_error_handler( parser, NULL );
     syck_parser_implicit_typing( parser, 0 );
@@ -115,7 +112,7 @@ rb_syck_compile(self, port)
     oid = syck_parse( parser );
     syck_lookup_sym( parser, oid, (char **)&sav );
 
-    ret = S_ALLOCA_N( char, strlen( sav->buffer ) + 3 );
+    ret = S_ALLOC_N( char, strlen( sav->buffer ) + 3 );
     ret[0] = '\0';
     strcat( ret, "D\n" );
     strcat( ret, sav->buffer );
@@ -164,12 +161,12 @@ rb_syck_io_str_read( char *buf, SyckIoStr *str, long max_size, long skip )
  * (returns tainted? boolean)
  */
 int
-syck_parser_assign_io(parser, pport)
+syck_parser_assign_io(parser, port)
     SyckParser *parser;
-    VALUE *pport;
+    VALUE port;
 {
     int taint = Qtrue;
-    VALUE tmp, port = *pport;
+    VALUE tmp;
     if (!NIL_P(tmp = rb_check_string_type(port))) {
         taint = OBJ_TAINTED(port); /* original taintedness */
         port = tmp;
@@ -184,7 +181,6 @@ syck_parser_assign_io(parser, pport)
     else {
         rb_raise(rb_eTypeError, "instance of IO needed");
     }
-    *pport = port;
     return taint;
 }
 
@@ -266,11 +262,12 @@ rb_syck_mktime(str, len)
     ptr += 2;
     if ( len > ptr - str && *ptr == '.' )
     {
-        char padded[] = "000000";
+        char *padded = syck_strndup( "000000", 6 );
         char *end = ptr + 1;
         while ( isdigit( *end ) ) end++;
         MEMCPY(padded, ptr + 1, char, end - (ptr + 1));
         usec = strtol(padded, NULL, 10);
+        S_FREE(padded);
     }
     else
     {
@@ -595,8 +592,6 @@ yaml_org_handler( n, ref )
     return transferred;
 }
 
-static void syck_node_mark( SyckNode *n );
-
 /*
  * {native mode} node handler
  * - Converts data into native Ruby types
@@ -705,13 +700,6 @@ syck_set_model( p, input, model )
     syck_parser_bad_anchor_handler( parser, rb_syck_bad_anchor_handler );
 }
 
-static int
-syck_st_mark_nodes( char *key, SyckNode *n, char *arg )
-{
-    if ( n != (void *)1 ) syck_node_mark( n );
-    return ST_CONTINUE;
-}
-
 /*
  * mark parser nodes
  */
@@ -719,20 +707,14 @@ static void
 syck_mark_parser(parser)
     SyckParser *parser;
 {
-    struct parser_xtra *bonus = (struct parser_xtra *)parser->bonus;
-    rb_gc_mark_maybe(parser->root);
-    rb_gc_mark_maybe(parser->root_on_error);
-    rb_gc_mark( bonus->data );
-    rb_gc_mark( bonus->proc );
-    rb_gc_mark( bonus->resolver );
-
-    if ( parser->anchors != NULL )
+    struct parser_xtra *bonus;
+    rb_gc_mark(parser->root);
+    rb_gc_mark(parser->root_on_error);
+    if ( parser->bonus != NULL )
     {
-        st_foreach( parser->anchors, syck_st_mark_nodes, 0 );
-    }
-    if ( parser->bad_anchors != NULL )
-    {
-        st_foreach( parser->bad_anchors, syck_st_mark_nodes, 0 );
+        bonus = (struct parser_xtra *)parser->bonus;
+        rb_gc_mark( bonus->data );
+        rb_gc_mark( bonus->proc );
     }
 }
 
@@ -743,7 +725,8 @@ void
 rb_syck_free_parser(p)
     SyckParser *p;
 {
-    S_FREE( p->bonus );
+    struct parser_xtra *bonus = (struct parser_xtra *)p->bonus;
+    if ( bonus != NULL ) S_FREE( bonus );
     syck_free_parser(p);
 }
 
@@ -758,9 +741,6 @@ syck_parser_s_alloc(class)
     VALUE pobj;
     SyckParser *parser = syck_new_parser();
 
-    parser->bonus = S_ALLOC( struct parser_xtra );
-    S_MEMZERO( parser->bonus, struct parser_xtra, 1 );
-
     pobj = Data_Wrap_Struct( class, syck_mark_parser, rb_syck_free_parser, parser );
 
     syck_parser_set_root_on_error( parser, Qnil );
@@ -787,7 +767,6 @@ syck_parser_initialize(argc, argv, self)
         Check_Type(options, T_HASH);
     }
     rb_ivar_set(self, s_options, options);
-    rb_ivar_set(self, s_input, Qnil);
     return self;
 }
 
@@ -832,7 +811,8 @@ syck_parser_load(argc, argv, self)
 {
     VALUE port, proc, model, input;
     SyckParser *parser;
-    struct parser_xtra *bonus;
+    struct parser_xtra *bonus = S_ALLOC_N( struct parser_xtra, 1 );
+    volatile VALUE hash;    /* protect from GC */
 
     rb_scan_args(argc, argv, "11", &port, &proc);
 
@@ -841,12 +821,13 @@ syck_parser_load(argc, argv, self)
     Data_Get_Struct(self, SyckParser, parser);
     syck_set_model( self, input, model );
 
-    bonus = (struct parser_xtra *)parser->bonus;
-    bonus->taint = syck_parser_assign_io(parser, &port);
-    bonus->data = rb_hash_new();
+    bonus->taint = syck_parser_assign_io(parser, port);
+    bonus->data = hash = rb_hash_new();
     bonus->resolver = rb_attr_get( self, s_resolver );
     if ( NIL_P( proc ) ) bonus->proc = 0;
     else                 bonus->proc = proc;
+    
+    parser->bonus = (void *)bonus;
 
     return syck_parse( parser );
 }
@@ -862,7 +843,8 @@ syck_parser_load_documents(argc, argv, self)
 {
     VALUE port, proc, v, input, model;
     SyckParser *parser;
-    struct parser_xtra *bonus;
+    struct parser_xtra *bonus = S_ALLOC_N( struct parser_xtra, 1 );
+    volatile VALUE hash;
 
     rb_scan_args(argc, argv, "1&", &port, &proc);
 
@@ -871,15 +853,15 @@ syck_parser_load_documents(argc, argv, self)
     Data_Get_Struct(self, SyckParser, parser);
     syck_set_model( self, input, model );
     
-    bonus = (struct parser_xtra *)parser->bonus;
-    bonus->taint = syck_parser_assign_io(parser, &port);
+    bonus->taint = syck_parser_assign_io(parser, port);
     bonus->resolver = rb_attr_get( self, s_resolver );
     bonus->proc = 0;
+    parser->bonus = (void *)bonus;
 
     while ( 1 )
     {
         /* Reset hash for tracking nodes */
-        bonus->data = rb_hash_new();
+        bonus->data = hash = rb_hash_new();
 
         /* Parse a document */
         v = syck_parse( parser );
@@ -1043,11 +1025,12 @@ syck_set_ivars( vars, obj )
     VALUE ivname = rb_ary_entry( vars, 0 );
     char *ivn;
     StringValue( ivname );
-    ivn = S_ALLOCA_N( char, RSTRING(ivname)->len + 2 );
+    ivn = S_ALLOC_N( char, RSTRING(ivname)->len + 2 );
     ivn[0] = '@';
     ivn[1] = '\0';
     strncat( ivn, RSTRING(ivname)->ptr, RSTRING(ivname)->len );
     rb_iv_set( obj, ivn, rb_ary_entry( vars, 1 ) );
+    S_FREE( ivn );
     return Qnil;
 }
 
@@ -1134,10 +1117,6 @@ syck_resolver_transfer( self, type, val )
                             type = subclass;
                             subclass = cYObject;
                         }
-                        else /* workaround for SEGV. real fix please */
-                        {
-                            rb_raise( rb_eTypeError, "invalid subclass" );
-                        }
                     }
                     break;
                 }
@@ -1160,15 +1139,7 @@ syck_resolver_transfer( self, type, val )
             }
             else if ( !NIL_P( target_class ) )
             {
-                if ( subclass == rb_cBignum )
-                {
-                    obj = rb_str2inum( val, 10 ); /* for yaml dumped by 1.8.3 [ruby-core:6159] */
-                }
-                else
-                {
-                    obj = rb_obj_alloc( subclass );
-                }
-
+                obj = rb_obj_alloc( subclass );
                 if ( rb_respond_to( obj, s_yaml_initialize ) )
                 {
                     rb_funcall( obj, s_yaml_initialize, 2, type, val );
@@ -1212,9 +1183,10 @@ syck_resolver_tagurize( self, val )
 
     if ( !NIL_P(tmp) )
     {
-        char *taguri = syck_type_id_to_uri( RSTRING(tmp)->ptr );
-        val = rb_str_new2( taguri );
-        S_FREE( taguri );
+        char *taguri;
+        val = tmp;
+        taguri = syck_type_id_to_uri( RSTRING(val)->ptr );
+        return rb_str_new2( taguri );
     }
 
     return val;
@@ -1364,7 +1336,7 @@ syck_badalias_cmp( alias1, alias2 )
  */
 VALUE
 syck_domaintype_initialize( self, domain, type_id, val )
-    VALUE self, domain, type_id, val;
+    VALUE self, type_id, val;
 {
     rb_iv_set( self, "@domain", domain );
     rb_iv_set( self, "@type_id", type_id );
@@ -1404,7 +1376,6 @@ syck_node_mark( n )
     SyckNode *n;
 {
     int i;
-    rb_gc_mark_maybe( n->id );
     switch ( n->kind )
     {
         case syck_seq_kind:
@@ -1422,9 +1393,42 @@ syck_node_mark( n )
             }
         break;
     }
-#if 0 /* maybe needed */
-    if ( n->shortcut ) syck_node_mark( n->shortcut ); /* caution: maybe cyclic */
-#endif
+}
+
+/*
+ * Don't free Ruby data, Ruby will do that
+ */
+void
+rb_syck_free_node( SyckNode *n )
+{
+    switch ( n->kind  )
+    {
+        case syck_str_kind:
+            S_FREE( n->data.str );
+            n->data.str = NULL;
+        break;
+
+        case syck_seq_kind:
+            if ( n->data.list != NULL )
+            {
+                S_FREE( n->data.list->items );
+                S_FREE( n->data.list );
+                n->data.list = NULL;
+            }
+        break;
+
+        case syck_map_kind:
+            if ( n->data.pairs != NULL )
+            {
+                S_FREE( n->data.pairs->keys );
+                S_FREE( n->data.pairs->values );
+                S_FREE( n->data.pairs );
+                n->data.pairs = NULL;
+            }
+        break;
+    }
+
+    S_FREE( n );
 }
 
 /*
@@ -1435,7 +1439,7 @@ syck_scalar_alloc( class )
     VALUE class;
 {
     SyckNode *node = syck_alloc_str();
-    VALUE obj = Data_Wrap_Struct( class, syck_node_mark, syck_free_node, node );
+    VALUE obj = Data_Wrap_Struct( class, syck_node_mark, rb_syck_free_node, node );
     node->id = obj;
     return obj;
 }
@@ -1504,7 +1508,7 @@ syck_scalar_value_set( self, val )
     Data_Get_Struct( self, SyckNode, node );
 
     StringValue( val );
-    node->data.str->ptr = syck_strndup( RSTRING(val)->ptr, RSTRING(val)->len );
+    node->data.str->ptr = RSTRING(val)->ptr;
     node->data.str->len = RSTRING(val)->len;
     node->data.str->style = scalar_none;
 
@@ -1522,7 +1526,7 @@ syck_seq_alloc( class )
     SyckNode *node;
     VALUE obj;
     node = syck_alloc_seq();
-    obj = Data_Wrap_Struct( class, syck_node_mark, syck_free_node, node );
+    obj = Data_Wrap_Struct( class, syck_node_mark, rb_syck_free_node, node );
     node->id = obj;
     return obj;
 }
@@ -1621,7 +1625,7 @@ syck_map_alloc( class )
     SyckNode *node;
     VALUE obj;
     node = syck_alloc_map();
-    obj = Data_Wrap_Struct( class, syck_node_mark, syck_free_node, node );
+    obj = Data_Wrap_Struct( class, syck_node_mark, rb_syck_free_node, node );
     node->id = obj;
     return obj;
 }
@@ -1751,7 +1755,8 @@ syck_node_init_copy( copy, orig )
     if ( copy == orig )
         return copy;
 
-    if ( TYPE( orig ) != T_DATA )
+    if ( TYPE( orig ) != T_DATA ||
+         RDATA( orig )->dfree != ( RUBY_DATA_FUNC )rb_syck_free_node )
     {
         rb_raise( rb_eTypeError, "wrong argument type" );
     }
@@ -1772,11 +1777,12 @@ syck_node_type_id_set( self, type_id )
     SyckNode *node;
     Data_Get_Struct( self, SyckNode, node );
 
-    S_FREE( node->type_id );
+    if ( node->type_id != NULL ) S_FREE( node->type_id );
 
-    if ( !NIL_P( type_id ) ) {
-        StringValue( type_id );
-        node->type_id = syck_strndup( RSTRING(type_id)->ptr, RSTRING(type_id)->len );
+    if ( NIL_P( type_id ) ) {
+        node->type_id = NULL;
+    } else {
+        node->type_id = StringValuePtr( type_id );
     }
 
     rb_iv_set( self, "@type_id", type_id );
@@ -1794,14 +1800,13 @@ syck_node_transform( self )
     SyckNode *n;
     SyckNode *orig_n;
     Data_Get_Struct(self, SyckNode, orig_n);
-    t = Data_Wrap_Struct( cNode, syck_node_mark, syck_free_node, 0 );
 
     switch (orig_n->kind)
     {
         case syck_map_kind:
             {
                 int i;
-                DATA_PTR(t) = n = syck_alloc_map();
+                n = syck_alloc_map();
                 for ( i = 0; i < orig_n->data.pairs->idx; i++ )
                 {
                     syck_map_add( n, rb_funcall( syck_map_read( orig_n, map_key, i ), s_transform, 0 ),
@@ -1813,7 +1818,7 @@ syck_node_transform( self )
         case syck_seq_kind:
             {
                 int i;
-                DATA_PTR(t) = n = syck_alloc_seq();
+                n = syck_alloc_seq();
                 for ( i = 0; i < orig_n->data.list->idx; i++ )
                 {
                     syck_seq_add( n, rb_funcall( syck_seq_read( orig_n, i ), s_transform, 0 ) );
@@ -1822,7 +1827,7 @@ syck_node_transform( self )
         break;
 
         case syck_str_kind:
-            DATA_PTR(t) = n = syck_new_str2( orig_n->data.str->ptr, orig_n->data.str->len, orig_n->data.str->style );
+            n = syck_new_str2( orig_n->data.str->ptr, orig_n->data.str->len, orig_n->data.str->style );
         break;
     }
 
@@ -1834,8 +1839,11 @@ syck_node_transform( self )
     {
         n->anchor = syck_strndup( orig_n->anchor, strlen( orig_n->anchor ) );
     }
+    t = Data_Wrap_Struct( cNode, NULL, NULL, n );
     n->id = t;
-    return rb_funcall( oDefaultResolver, s_node_import, 1, t );
+    t = rb_funcall( oDefaultResolver, s_node_import, 1, t );
+    syck_free_node( n );
+    return t;
 }
 
 /*
@@ -1930,10 +1938,13 @@ static void
 syck_mark_emitter(emitter)
     SyckEmitter *emitter;
 {
-    struct emitter_xtra *bonus = (struct emitter_xtra *)emitter->bonus;
-    rb_gc_mark( bonus->oid  );
-    rb_gc_mark( bonus->data );
-    rb_gc_mark( bonus->port );
+    struct emitter_xtra *bonus;
+    if ( emitter->bonus != NULL )
+    {
+        bonus = (struct emitter_xtra *)emitter->bonus;
+        rb_gc_mark( bonus->data );
+        rb_gc_mark( bonus->port );
+    }
 }
 
 /*
@@ -1943,7 +1954,8 @@ void
 rb_syck_free_emitter(e)
     SyckEmitter *e;
 {
-    S_FREE( e->bonus );
+    struct emitter_xtra *bonus = (struct emitter_xtra *)e->bonus;
+    if ( bonus != NULL ) S_FREE( bonus );
     syck_free_emitter(e);
 }
 
@@ -1958,9 +1970,6 @@ syck_emitter_s_alloc(class)
     VALUE pobj;
     SyckEmitter *emitter = syck_new_emitter();
 
-    emitter->bonus = S_ALLOC( struct emitter_xtra );
-    S_MEMZERO( emitter->bonus, struct emitter_xtra, 1 );
-
     pobj = Data_Wrap_Struct( class, syck_mark_emitter, rb_syck_free_emitter, emitter );
     syck_emitter_handler( emitter, rb_syck_emitter_handler );
     syck_output_handler( emitter, rb_syck_output_handler );
@@ -1981,13 +1990,15 @@ syck_emitter_reset( argc, argv, self )
     VALUE options, tmp;
     SyckEmitter *emitter;
     struct emitter_xtra *bonus;
+    volatile VALUE hash;    /* protect from GC */
 
     Data_Get_Struct(self, SyckEmitter, emitter);
     bonus = (struct emitter_xtra *)emitter->bonus;
+    if ( bonus != NULL ) S_FREE( bonus );
 
-    bonus->oid = Qnil;
+    bonus = S_ALLOC_N( struct emitter_xtra, 1 );
     bonus->port = rb_str_new2( "" );
-    bonus->data = rb_hash_new();
+    bonus->data = hash = rb_hash_new();
 
     if (rb_scan_args(argc, argv, "01", &options) == 0)
     {
@@ -2008,7 +2019,8 @@ syck_emitter_reset( argc, argv, self )
         rb_ivar_set(self, s_options, options);
     }
     
-    emitter->headless = 0;
+    emitter->headless = 1;
+    emitter->bonus = (void *)bonus;
     rb_ivar_set(self, s_level, INT2FIX(0));
     rb_ivar_set(self, s_resolver, Qnil);
     return self;
@@ -2236,12 +2248,10 @@ Init_syck()
     rb_define_method( cResolver, "node_import", syck_resolver_node_import, 1 );
     rb_define_method( cResolver, "tagurize", syck_resolver_tagurize, 1 );
 
-    rb_global_variable( &oDefaultResolver );
     oDefaultResolver = rb_funcall( cResolver, rb_intern( "new" ), 0 );
     rb_define_singleton_method( oDefaultResolver, "node_import", syck_defaultresolver_node_import, 1 );
     rb_define_singleton_method( oDefaultResolver, "detect_implicit", syck_defaultresolver_detect_implicit, 1 );
     rb_define_const( rb_syck, "DefaultResolver", oDefaultResolver );
-    rb_global_variable( &oGenericResolver );
     oGenericResolver = rb_funcall( cResolver, rb_intern( "new" ), 0 );
     rb_define_singleton_method( oGenericResolver, "node_import", syck_genericresolver_node_import, 1 );
     rb_define_const( rb_syck, "GenericResolver", oGenericResolver );
