@@ -1,8 +1,8 @@
 /*
  * syck.h
  *
- * $Author: nobu $
- * $Date: 2007-07-15 22:24:51 +0900 (Sun, 15 Jul 2007) $
+ * $Author: why $
+ * $Date: 2005/09/20 06:46:44 $
  *
  * Copyright (C) 2003 why the lucky stiff
  */
@@ -17,7 +17,6 @@
 #define YAML_DOMAIN     "yaml.org,2002"
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <ctype.h>
 #include "st.h"
 
@@ -52,7 +51,7 @@ extern "C" {
 #define S_ALLOC_N(type,n) (type*)malloc(sizeof(type)*(n))
 #define S_ALLOC(type) (type*)malloc(sizeof(type))
 #define S_REALLOC_N(var,type,n) (var)=(type*)realloc((char*)(var),sizeof(type)*(n))
-#define S_FREE(n) if (n) { free(n); n = NULL; }
+#define S_FREE(n) free(n); n = NULL;
 
 #define S_ALLOCA_N(type,n) (type*)alloca(sizeof(type)*(n))
 
@@ -400,7 +399,6 @@ int syck_scan_scalar( int, char *, long );
 void syck_parser_handler( SyckParser *, SyckNodeHandler );
 void syck_parser_error_handler( SyckParser *, SyckErrorHandler );
 void syck_parser_bad_anchor_handler( SyckParser *, SyckBadAnchorHandler );
-void syck_parser_set_input_type( SyckParser *, enum syck_parser_input );
 void syck_parser_file( SyckParser *, FILE *, SyckIoFileRead );
 void syck_parser_str( SyckParser *, char *, long, SyckIoStrRead );
 void syck_parser_str_auto( SyckParser *, char *, SyckIoStrRead );
@@ -447,9 +445,6 @@ long syck_seq_count( SyckNode * );
  * Lexer prototypes
  */
 void syckerror( char * );
-int syckparse( void * );
-union YYSTYPE;
-int sycklex( union YYSTYPE *, SyckParser * );
 
 #if defined(__cplusplus)
 }  /* extern "C" { */
