@@ -44,19 +44,6 @@ module Tk::BLT::Treeview::ConfigMethod
   end
   private :__item_boolval_optkeys
 
-  def __item_strval_optkeys(id)
-    case id
-    when Array
-      # id := [ 'column', name ]
-      super() << 'titleforeground' << 'titleshadow'
-    when 'sort'
-      ['decreasing']
-    else
-      []
-    end
-  end
-  private :__item_strval_optkeys
-
   def __item_listval_optkeys(id)
     case id
     when 'entry'
@@ -98,9 +85,6 @@ module Tk::BLT::Treeview::ConfigMethod
   def column_cget(name, option)
     itemcget(['column', name], option)
   end
-  def column_cget_strict(name, option)
-    itemcget_strict(['column', name], option)
-  end
   def column_configure(name, slot, value=None)
     itemconfigure(['column', name], slot, value)
   end
@@ -114,9 +98,6 @@ module Tk::BLT::Treeview::ConfigMethod
   def button_cget(option)
     itemcget('button', option)
   end
-  def button_cget_strict(option)
-    itemcget_strict('button', option)
-  end
   def button_configure(slot, value=None)
     itemconfigure('button', slot, value)
   end
@@ -135,14 +116,6 @@ module Tk::BLT::Treeview::ConfigMethod
       ret
     end
   end
-  def entry_cget_strict(option)
-    ret = itemcget_strict('entry', option)
-    if option == 'bindtags' || option == :bindtags
-      ret.collect{|tag| TkBindTag.id2obj(tag)}
-    else
-      ret
-    end
-  end
   def entry_configure(slot, value=None)
     itemconfigure('entry', slot, value)
   end
@@ -184,9 +157,6 @@ module Tk::BLT::Treeview::ConfigMethod
   def sort_cget(option)
     itemcget('sort', option)
   end
-  def sort_cget_strict(option)
-    itemcget_strict('sort', option)
-  end
   def sort_configure(slot, value=None)
     itemconfigure('sort', slot, value)
   end
@@ -200,9 +170,6 @@ module Tk::BLT::Treeview::ConfigMethod
   def text_cget(option)
     itemcget('text', option)
   end
-  def text_cget_strict(option)
-    itemcget_strict('text', option)
-  end
   def text_configure(slot, value=None)
     itemconfigure('text', slot, value)
   end
@@ -213,8 +180,8 @@ module Tk::BLT::Treeview::ConfigMethod
     current_itemconfiginfo('text', slot)
   end
 
-  private :itemcget, :itemcget_strict
-  private :itemconfigure, :itemconfiginfo, :current_itemconfiginfo
+  private :itemcget, :itemconfigure
+  private :itemconfiginfo, :current_itemconfiginfo
 end
 
 class Tk::BLT::Treeview
@@ -230,16 +197,11 @@ class Tk::BLT::Treeview
   ########################
 
   def __boolval_optkeys
-    ['autocreate', 'allowduplicates', 'exportselection', 'flat', 'hideroot', 
+    ['autocreate', 'exportselection', 'flat', 'hideroot', 
       'newtags', 'showtitles', 'sortselection']
   end
   private :__boolval_optkeys
 
-  def __strval_optkeys
-    super() + ['focusforeground', 'linecolor', 'separator', 'trim']
-  end
-  private :__strval_optkeys
-
   ########################
 
   class OpenCloseCommand < TkValidateCommand
@@ -259,24 +221,6 @@ class Tk::BLT::Treeview
         nil
       ]
 
-=begin
-      # for Ruby m17n :: ?x --> String --> char-code ( getbyte(0) )
-      KEY_TBL.map!{|inf|
-        if inf.kind_of?(Array)
-          inf[0] = inf[0].getbyte(0) if inf[0].kind_of?(String)
-          inf[1] = inf[1].getbyte(0) if inf[1].kind_of?(String)
-        end
-        inf
-      }
-
-      PROC_TBL.map!{|inf|
-        if inf.kind_of?(Array)
-          inf[0] = inf[0].getbyte(0) if inf[0].kind_of?(String)
-        end
-        inf
-      }
-=end
-
       _setup_subst_table(KEY_TBL, PROC_TBL);
 
       def self.ret_val(val)
@@ -311,12 +255,8 @@ class Tk::BLT::Treeview
   ########################
 
   def __destroy_hook__
-    Tk::BLT::Treeview::Node::TreeNodeID_TBL.mutex.synchronize{
-      Tk::BLT::Treeview::Node::TreeNodeID_TBL.delete(@path)
-    }
-    Tk::BLT::Treeview::Tag::TreeTagID_TBL.mutex.synchronize{
-      Tk::BLT::Treeview::Tag::TreeTagID_TBL.delete(@path)
-    }
+    Tk::BLT::Treeview::Node::TreeNodeID_TBL.delete(@path)
+    Tk::BLT::Treeview::Tag::TreeTagID_TBL.delete(@path)
   end
 
   def tagid(tag)
@@ -514,24 +454,6 @@ class Tk::BLT::Treeview
         nil
       ]
 
-=begin
-      # for Ruby m17n :: ?x --> String --> char-code ( getbyte(0) )
-      KEY_TBL.map!{|inf|
-        if inf.kind_of?(Array)
-          inf[0] = inf[0].getbyte(0) if inf[0].kind_of?(String)
-          inf[1] = inf[1].getbyte(0) if inf[1].kind_of?(String)
-        end
-        inf
-      }
-
-      PROC_TBL.map!{|inf|
-        if inf.kind_of?(Array)
-          inf[0] = inf[0].getbyte(0) if inf[0].kind_of?(String)
-        end
-        inf
-      }
-=end
-
       _setup_subst_table(KEY_TBL, PROC_TBL);
 
       def self.ret_val(val)
@@ -547,8 +469,7 @@ class Tk::BLT::Treeview
   def _find_exec_flag_value(val)
     if val.kind_of?(Array)
       cmd, *args = val
-      #FindExecFlagValue.new(cmd, args.join(' '))
-      FindExecFlagValue.new(cmd, *args)
+      FindExecFlagValue.new(cmd, args.join(' '))
     elsif TkComm._callback_entry?(val)
       FindExecFlagValue.new(val)
     else
@@ -1028,47 +949,22 @@ class Tk::BLT::Treeview::Node < TkObject
   include Tk::BLT::Treeview::TagOrID_Methods
 
   TreeNodeID_TBL = TkCore::INTERP.create_table
+  TreeNode_ID = ['blt_treeview_node'.freeze, '00000'.taint].freeze
 
-  (TreeNode_ID = ['blt_treeview_node'.freeze, '00000'.taint]).instance_eval{
-    @mutex = Mutex.new
-    def mutex; @mutex; end
-    freeze
-  }
-
-  TkCore::INTERP.init_ip_env{
-    TreeNodeID_TBL.mutex.synchronize{ TreeNodeID_TBL.clear }
-  }
+  TkCore::INTERP.init_ip_env{ TreeNodeID_TBL.clear }
 
   def self.id2obj(tree, id)
     tpath = tree.path
-    TreeNodeID_TBL.mutex.synchronize{
-      if TreeNodeID_TBL[tpath]
-        if TreeNodeID_TBL[tpath][id]
-          TreeNodeID_TBL[tpath][id]
-        else
-          begin
-            # self.new(tree, nil, nil, 'node'=>Integer(id))
-            unless (tk_call(@tpath, 'get', id)).empty?
-              id = Integer(id)
-              (obj = self.allocate).instance_eval{
-                @parent = @tree = tree
-                @tpath = @parent.path
-                @path = @id = id
-                TreeNodeID_TBL[@tpath] ||= {}
-                TreeNodeID_TBL[@tpath][@id] = self
-              }
-              obj
-            else
-              id
-            end
-          rescue
-            id
-          end
-        end
-      else
+    return id unless TreeNodeID_TBL[tpath]
+    if TreeNodeID_TBL[tpath][id]
+      TreeNodeID_TBL[tpath][id]
+    else
+      begin
+        self.new(tree, nil, nil, 'node'=>Integer(id))
+      rescue
         id
       end
-    }
+    end
   end
 
   def self.new(tree, pos, parent=nil, keys={})
@@ -1080,21 +976,13 @@ class Tk::BLT::Treeview::Node < TkObject
     keys = _symbolkey2str(keys)
     tpath = tree.path
 
-    TreeNodeID_TBL.mutex.synchronize{
-      TreeNodeID_TBL[tpath] ||= {}
-      if (id = keys['node']) && (obj = TreeNodeID_TBL[tpath][id])
-        keys.delete('node')
-        tk_call(tree.path, 'move', id, pos, parent) if parent
-        return obj
-      end
+    if (id = keys['node']) && (obj = TreeNodeID_TBL[tpath][id])
+      keys.delete('node')
+      tk_call(tree.path, 'move', id, pos, parent) if parent
+      return obj
+    end
 
-      #super(tree, pos, parent, keys)
-      (obj = self.allocate).instance_eval{
-        initialize(tree, pos, parent, keys)
-        TreeNodeID_TBL[tpath][@id] = self
-      }
-      obj
-    }
+    super(tree, pos, parent, keys)
   end
 
   def initialize(tree, pos, parent, keys)
@@ -1102,18 +990,11 @@ class Tk::BLT::Treeview::Node < TkObject
     @tpath = @parent.path
 
     if (id = keys['node'])
-      # if tk_call(@tpath, 'get', id).empty?
-      #   fail RuntimeError, "not exist the node '#{id}'"
-      # end
       @path = @id = id
       tk_call(@tpath, 'move', @id, pos, tagid(parent)) if parent
-      configure(keys) if keys && ! keys.empty?
     else
-      name = nil
-      TreeNode_ID.mutex.synchronize{
-        name = TreeNode_ID.join(TkCore::INTERP._ip_id_).freeze
-        TreeNode_ID[1].succ!
-      }
+      name = TreeNode_ID.join(TkCore::INTERP._ip_id_).freeze
+      TreeNode_ID[1].succ!
 
       at = keys.delete['at']
 
@@ -1136,6 +1017,9 @@ class Tk::BLT::Treeview::Node < TkObject
       end
       @path = @id
     end
+
+    TreeNodeID_TBL[@tpath] = {} unless TreeNodeID_TBL[@tpath]
+    TreeNodeID_TBL[@tpath][@id] = self
   end
 
   def id
@@ -1149,66 +1033,37 @@ class Tk::BLT::Treeview::Tag < TkObject
   include Tk::BLT::Treeview::TagOrID_Methods
 
   TreeTagID_TBL = TkCore::INTERP.create_table
+  TreeTag_ID = ['blt_treeview_tag'.freeze, '00000'.taint].freeze
 
-  (TreeTag_ID = ['blt_treeview_tag'.freeze, '00000'.taint]).instance_eval{
-    @mutex = Mutex.new
-    def mutex; @mutex; end
-    freeze
-  }
+  TkCore::INTERP.init_ip_env{ TreeTagID_TBL.clear }
 
-  TkCore::INTERP.init_ip_env{
-    TreeTagID_TBL.mutex.synchronize{ TreeTagID_TBL.clear }
-  }
-
-  def self.id2obj(tree, name)
+  def self.id2obj(tree, id)
     tpath = tree.path
-    TreeTagID_TBL.mutex.synchronize{
-      if TreeTagID_TBL[tpath]
-        if TreeTagID_TBL[tpath][name]
-          TreeTagID_TBL[tpath][name]
-        else
-          #self.new(tree, name)
-          (obj = self.allocate).instance_eval{
-            @parent = @tree = tree
-            @tpath = @parent.path
-            @path = @id = name
-            TreeTagID_TBL[@tpath] = {} unless TreeTagID_TBL[@tpath]
-            TreeTagID_TBL[@tpath][@id] = self
-          }
-          obj
-        end
-      else
+    return id unless TreeTagID_TBL[tpath]
+    if TreeTagID_TBL[tpath][id]
+      TreeTagID_TBL[tpath][id]
+    else
+      begin
+        self.new(tree, nil, nil, 'name'=>Integer(id))
+      rescue
         id
       end
-    }
+    end
   end
 
   def self.new_by_name(tree, name, *ids)
-    TreeTagID_TBL.mutex.synchronize{
-      unless (obj = TreeTagID_TBL[tree.path][name])
-        (obj = self.allocate).instance_eval{
-          initialize(tree, name, ids)
-          TreeTagID_TBL[@tpath] = {} unless TreeTagID_TBL[@tpath]
-          TreeTagID_TBL[@tpath][@id] = self
-        }
-      end
-      obj
-    }
+    if (obj = TreeTagID_TBL[tree.path][name])
+      return obj
+    end
+    new([tree, name], ids)
   end
 
   def self.new(tree, *ids)
-    TreeTagID_TBL.mutex.synchronize{
-      (obj = self.allocate).instance_eval{
-        if tree.kind_of?(Array)
-          initialize(tree[0], tree[1], ids)
-        else
-          initialize(tree, nil, ids)
-        end
-        TreeTagID_TBL[@tpath] = {} unless TreeTagID_TBL[@tpath]
-        TreeTagID_TBL[@tpath][@id] = self
-      }
-      obj
-    }
+    if tree.kind_of?(Array)
+      super(tree[0], tree[1], ids)
+    else
+      super(tree, nil, ids)
+    end
   end
 
   def initialize(tree, name, ids)
@@ -1218,12 +1073,13 @@ class Tk::BLT::Treeview::Tag < TkObject
     if name
       @path = @id = name
     else
-      TreeTag_ID.mutex.synchronize{
-        @path = @id = TreeTag_ID.join(TkCore::INTERP._ip_id_).freeze
-        TreeTag_ID[1].succ!
-      }
+      @path = @id = TreeTag_ID.join(TkCore::INTERP._ip_id_).freeze
+      TreeTag_ID[1].succ!
     end
 
+    TreeTagID_TBL[@tpath] = {} unless TreeTagID_TBL[@tpath]
+    TreeTagID_TBL[@tpath][@id] = self
+
     unless ids.empty?
       tk_call(@tpath, 'tag', 'add', @id, *(ids.collect{|id| tagid(id)}))
     end
