@@ -115,11 +115,6 @@ module Tk
 
       include Wm
       include TkMenuSpec
-
-      def __strval_optkeys
-        super() << 'title'
-      end
-      private :__strval_optkeys
     end
 
     ############################
@@ -145,16 +140,9 @@ module Tk
       private :__config_cmd
 
       ComponentID_TBL = TkCore::INTERP.create_table
+      Itk_Component_ID = ['itk:component'.freeze, '00000'.taint].freeze
 
-      (Itk_Component_ID = ['itk:component'.freeze, '00000'.taint]).instance_eval{
-        @mutex = Mutex.new
-        def mutex; @mutex; end
-        freeze
-      }
-
-      TkCore::INTERP.init_ip_env{
-        ComponentID_TBL.mutex.synchronize{ ComponentID_TBL.clear }
-      }
+      TkCore::INTERP.init_ip_env{ ComponentID_TBL.clear }
 
       def self.id2obj(master, id)
         if master.kind_of?(TkObject)
@@ -162,13 +150,8 @@ module Tk
         else
           master = master.to_s
         end
-        ComponentID_TBL.mutex.synchronize{
-          if ComponentID_TBL.key?(master)
-            (ComponentID_TBL[master].key?(id))? ComponentID_TBL[master][id]: id
-          else
-            id
-          end
-        }
+        return id unless ComponentID_TBL.key?(master)
+        (ComponentID_TBL.key?(id))? ComponentID_TBL[master][id]: id
       end
 
       def self.new(master, component=nil)
@@ -183,21 +166,17 @@ module Tk
         elsif component
           component = component.to_s
         else
-          Itk_Component_ID.mutex.synchronize{
-            component = Itk_Component_ID.join(TkCore::INTERP._ip_id_)
-            Itk_Component_ID[1].succ!
-          }
+          component = Itk_Component_ID.join(TkCore::INTERP._ip_id_)
+          Itk_Component_ID[1].succ!
         end
 
-        ComponentID_TBL.mutex.synchronize{
-          if ComponentID_TBL.key?(master)
-            if ComponentID_TBL[master].key?(component)
-              return ComponentID_TBL[master][component] 
-            end
-          else
-            ComponentID_TBL[master] = {}
+        if ComponentID_TBL.key?(master)
+          if ComponentID_TBL[master].key?(component)
+            return ComponentID_TBL[master][component] 
           end
-        }
+        else
+          ComponentID_TBL[master] = {}
+        end
 
         super(master, component)
       end
@@ -206,9 +185,7 @@ module Tk
         @master = master
         @component = component
 
-        ComponentID_TBL.mutex.synchronize{
-          ComponentID_TBL[@master][@component] = self
-        }
+        ComponentID_TBL[@master][@component] = self
 
         begin
           @widget = window(tk_call(@master, 'component', @component))
@@ -307,8 +284,7 @@ module Tk
         end
 
         # unknown method
-        super(id, *args)
-        # fail RuntimeError, "unknown method '#{name}' for #{self.inspect}"
+        fail RuntimeError, "unknown method '#{name}' for #{self.inspect}"
       end
 
       def tk_send(cmd, *rest)
