@@ -2,8 +2,8 @@
 
   gc.c -
 
-  $Author: akr $
-  $Date: 2004/12/20 13:53:34 $
+  $Author: matz $
+  $Date: 2005/07/27 14:29:11 $
   created at: Tue Oct  5 09:44:46 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -185,14 +185,14 @@ static st_table *finalizer_table = 0;
 /*
  *  call-seq:
  *     GC.enable    => true or false
- *  
+ *
  *  Enables garbage collection, returning <code>true</code> if garbage
  *  collection was previously disabled.
- *     
+ *
  *     GC.disable   #=> false
  *     GC.enable    #=> true
  *     GC.enable    #=> false
- *     
+ *
  */
 
 VALUE
@@ -207,13 +207,13 @@ rb_gc_enable()
 /*
  *  call-seq:
  *     GC.disable    => true or false
- *  
+ *
  *  Disables garbage collection, returning <code>true</code> if garbage
  *  collection was already disabled.
- *     
+ *
  *     GC.disable   #=> false
  *     GC.disable   #=> true
- *     
+ *
  */
 
 VALUE
@@ -296,7 +296,7 @@ typedef struct RVALUE {
 	struct RFile   file;
 	struct RNode   node;
 	struct RMatch  match;
-	struct RVarmap varmap; 
+	struct RVarmap varmap;
 	struct SCOPE   scope;
     } as;
 #ifdef GC_DEBUG
@@ -398,7 +398,7 @@ rb_data_object_alloc(klass, datap, dmark, dfree)
     RUBY_DATA_FUNC dfree;
 {
     NEWOBJ(data, struct RData);
-    Check_Type(klass, T_CLASS);
+    if (klass) Check_Type(klass, T_CLASS);
     OBJSETUP(data, klass, T_DATA);
     data->data = datap;
     data->dfree = dfree;
@@ -619,9 +619,11 @@ mark_locations_array(x, n)
     register VALUE *x;
     register long n;
 {
+    VALUE v;
     while (n--) {
-	if (is_pointer_to_heap((void *)*x)) {
-	    gc_mark(*x, 0);
+        v = *x;
+	if (is_pointer_to_heap((void *)v)) {
+	    gc_mark(v, 0);
 	}
 	x++;
     }
@@ -701,7 +703,7 @@ rb_gc_mark_maybe(obj)
 
 #define GC_LEVEL_MAX 250
 
-void
+static void
 gc_mark(ptr, lev)
     VALUE ptr;
     int lev;
@@ -711,7 +713,7 @@ gc_mark(ptr, lev)
     obj = RANY(ptr);
     if (rb_special_const_p(ptr)) return; /* special const not marked */
     if (obj->as.basic.flags == 0) return;       /* free cell */
-    if (obj->as.basic.flags & FL_MARK) return;  /* already marked */ 
+    if (obj->as.basic.flags & FL_MARK) return;  /* already marked */
     obj->as.basic.flags |= FL_MARK;
 
     if (lev > GC_LEVEL_MAX || (lev == 0 && ruby_stack_check())) {
@@ -749,7 +751,7 @@ gc_mark_children(ptr, lev)
     obj = RANY(ptr);
     if (rb_special_const_p(ptr)) return; /* special const not marked */
     if (obj->as.basic.flags == 0) return;       /* free cell */
-    if (obj->as.basic.flags & FL_MARK) return;  /* already marked */ 
+    if (obj->as.basic.flags & FL_MARK) return;  /* already marked */
     obj->as.basic.flags |= FL_MARK;
 
   marking:
@@ -1254,8 +1256,8 @@ obj_free(obj)
 	break;
 
       default:
-	rb_bug("gc_sweep(): unknown data type 0x%lx(%ld)", obj,
-	       RANY(obj)->as.basic.flags & T_MASK);
+	rb_bug("gc_sweep(): unknown data type 0x%lx(0x%lx)",
+	       RANY(obj)->as.basic.flags & T_MASK, obj);
     }
 }
 
@@ -1328,10 +1330,10 @@ garbage_collect()
     during_gc++;
 
     init_mark_stack();
-    
+
     /* mark frame stack */
     for (frame = ruby_frame; frame; frame = frame->prev) {
-	rb_gc_mark_frame(frame); 
+	rb_gc_mark_frame(frame);
 	if (frame->tmp) {
 	    struct FRAME *tmp = frame->tmp;
 	    while (tmp) {
@@ -1398,7 +1400,7 @@ garbage_collect()
     rb_mark_generic_ivar_tbl();
 
     rb_gc_mark_parser();
-    
+
     /* gc_mark objects whose marking are not completed*/
     while (!MARK_STACK_EMPTY){
 	if (mark_stack_overflow){
@@ -1425,7 +1427,7 @@ rb_gc()
  *     ObjectSpace.garbage_collect  => nil
  *
  *  Initiates garbage collection, unless manually disabled.
- *     
+ *
  */
 
 VALUE
@@ -1481,14 +1483,6 @@ Init_stack(addr)
 	    STACK_LEVEL_MAX = (rlim.rlim_cur - space) / sizeof(VALUE);
 	}
     }
-#if defined(__ia64__) && (!defined(__GNUC__) || __GNUC__ < 2 || defined(__OPTIMIZE__))
-    /* ruby crashes on IA64 if compiled with optimizer on */
-    /* when if STACK_LEVEL_MAX is greater than this magic number */
-    /* I know this is a kludge.  I suspect optimizer bug */
-#define IA64_MAGIC_STACK_LIMIT 49152
-    if (STACK_LEVEL_MAX > IA64_MAGIC_STACK_LIMIT)
-	STACK_LEVEL_MAX = IA64_MAGIC_STACK_LIMIT;
-#endif
 #endif
 }
 
@@ -1499,29 +1493,29 @@ Init_stack(addr)
  *  The <code>ObjectSpace</code> module contains a number of routines
  *  that interact with the garbage collection facility and allow you to
  *  traverse all living objects with an iterator.
- *     
+ *
  *  <code>ObjectSpace</code> also provides support for object
  *  finalizers, procs that will be called when a specific object is
  *  about to be destroyed by garbage collection.
- *     
+ *
  *     include ObjectSpace
- *     
- *     
+ *
+ *
  *     a = "A"
  *     b = "B"
  *     c = "C"
- *     
- *     
+ *
+ *
  *     define_finalizer(a, proc {|id| puts "Finalizer one on #{id}" })
  *     define_finalizer(a, proc {|id| puts "Finalizer two on #{id}" })
  *     define_finalizer(b, proc {|id| puts "Finalizer three on #{id}" })
- *     
+ *
  *  <em>produces:</em>
- *     
+ *
  *     Finalizer three on 537763470
  *     Finalizer one on 537763480
  *     Finalizer two on 537763480
- *     
+ *
  */
 
 void
@@ -1603,7 +1597,7 @@ os_obj_of(of)
 /*
  *  call-seq:
  *     ObjectSpace.each_object([module]) {|obj| ... } => fixnum
- *  
+ *
  *  Calls the block once for each living, nonimmediate object in this
  *  Ruby process. If <i>module</i> is specified, calls the block
  *  for only those classes or modules that match (or are a subclass of)
@@ -1613,15 +1607,15 @@ os_obj_of(of)
  *  never returned. In the example below, <code>each_object</code>
  *  returns both the numbers we defined and several constants defined in
  *  the <code>Math</code> module.
- *     
+ *
  *     a = 102.7
  *     b = 95       # Won't be returned
  *     c = 12345678987654321
  *     count = ObjectSpace.each_object(Numeric) {|x| p x }
  *     puts "Total count: #{count}"
- *     
+ *
  *  <em>produces:</em>
- *     
+ *
  *     12345678987654321
  *     102.7
  *     2.71828182845905
@@ -1630,7 +1624,7 @@ os_obj_of(of)
  *     1.7976931348623157e+308
  *     2.2250738585072e-308
  *     Total count: 7
- *     
+ *
  */
 
 static VALUE
@@ -1706,9 +1700,9 @@ call_final(os, obj)
 /*
  *  call-seq:
  *     ObjectSpace.undefine_finalizer(obj)
- *  
+ *
  *  Removes all finalizers for <i>obj</i>.
- *     
+ *
  */
 
 static VALUE
@@ -1724,10 +1718,10 @@ undefine_final(os, obj)
 /*
  *  call-seq:
  *     ObjectSpace.define_finalizer(obj, aProc=proc())
- *  
- *  Adds <i>aProc</i> as a finalizer, to be called when <i>obj</i>
- *  is about to be destroyed.
- *     
+ *
+ *  Adds <i>aProc</i> as a finalizer, to be called after <i>obj</i>
+ *  was destroyed.
+ *
  */
 
 static VALUE
@@ -1791,19 +1785,22 @@ run_final(obj)
 {
     long i;
     int status, critical_save = rb_thread_critical;
-    VALUE args[3], table;
+    VALUE args[3], table, objid;
 
+    objid = rb_obj_id(obj);	/* make obj into id */
     rb_thread_critical = Qtrue;
-    args[1] = rb_ary_new3(1, rb_obj_id(obj)); /* make obj into id */
+    args[1] = 0;
     args[2] = (VALUE)ruby_safe_level;
     for (i=0; i<RARRAY(finalizers)->len; i++) {
 	args[0] = RARRAY(finalizers)->ptr[i];
+	if (!args[1]) args[1] = rb_ary_new3(1, objid);
 	rb_protect((VALUE(*)_((VALUE)))run_single_final, (VALUE)args, &status);
     }
     if (finalizer_table && st_delete(finalizer_table, (st_data_t*)&obj, &table)) {
 	for (i=0; i<RARRAY(table)->len; i++) {
 	    VALUE final = RARRAY(table)->ptr[i];
 	    args[0] = RARRAY(final)->ptr[1];
+	    if (!args[1]) args[1] = rb_ary_new3(1, objid);
 	    args[2] = FIX2INT(RARRAY(final)->ptr[0]);
 	    rb_protect((VALUE(*)_((VALUE)))run_single_final, (VALUE)args, &status);
 	}
@@ -1831,7 +1828,9 @@ rb_gc_call_finalizer_at_exit()
 
     /* run finalizers */
     if (need_call_final) {
-	finalize_list(deferred_final_list);
+	p = deferred_final_list;
+	deferred_final_list = 0;
+	finalize_list(p);
 	for (i = 0; i < heaps_used; i++) {
 	    p = heaps[i].slot; pend = p + heaps[i].limit;
 	    while (p < pend) {
@@ -1870,14 +1869,14 @@ rb_gc_call_finalizer_at_exit()
 /*
  *  call-seq:
  *     ObjectSpace._id2ref(object_id) -> an_object
- *  
+ *
  *  Converts an object id to a reference to the object. May not be
  *  called on an object id passed as a parameter to a finalizer.
- *     
+ *
  *     s = "I am a string"                    #=> "I am a string"
  *     r = ObjectSpace._id2ref(s.object_id)   #=> "I am a string"
  *     r == s                                 #=> true
- *     
+ *
  */
 
 static VALUE
