@@ -13,6 +13,7 @@
 #   	by Keiju ISHITSUKA (Nippon Rational Inc.)
 #
 
+require "tracer"
 require "e2mmap"
 require "irb/slex"
 
@@ -435,7 +436,6 @@ class RubyLex
   def_exception(:SyntaxError, "%s")
   
   include RubyToken
-  include IRB
 
   attr_reader :continue
   attr_reader :lex_state
@@ -560,7 +560,7 @@ class RubyLex
     "q" => "\'",
     "Q" => "\"",
     "x" => "\`",
-    "r" => "/",
+    "r" => "\/",
     "w" => "]"
   }
   
@@ -575,7 +575,7 @@ class RubyLex
     "\'" => TkSTRING,
     "\"" => TkSTRING,
     "\`" => TkXSTRING,
-    "/" => TkREGEXP,
+    "\/" => TkREGEXP,
     "]" => TkDSTRING
   }
   Ltype2Token.default = TkSTRING
@@ -583,7 +583,7 @@ class RubyLex
   DLtype2Token = {
     "\"" => TkDSTRING,
     "\`" => TkDXSTRING,
-    "/" => TkDREGEXP,
+    "\/" => TkDREGEXP,
   }
 
   def lex_init()
@@ -1334,7 +1334,7 @@ class RubyLex
 	end
       end
 
-    when "C", "c" #, "^"
+    when "C", "c", "^"
       res << ch
       if ch == "C" and (ch = getc) != "-"
 	ungetc
@@ -1426,23 +1426,16 @@ module RDoc
 
     private 
 
-    def make_message(msg)
+    def warn(msg)
       prefix = "\n" + @input_file_name + ":"
       if @scanner
         prefix << "#{@scanner.line_no}:#{@scanner.char_no}: "
       end
-      return prefix + msg
-    end
-
-    def warn(msg)
-      return if @options.quiet
-      msg = make_message msg
-      $stderr.puts msg
+      $stderr.puts prefix + msg
     end
 
     def error(msg)
-      msg = make_message msg
-      $stderr.puts msg
+      warn msg
       exit(1)
     end
 
@@ -2558,7 +2551,7 @@ module RDoc
 	    break
 	  when TkCOMMA
 	  else
-           warn("unexpected token: '#{tk2.inspect}'") if $DEBUG
+	    warn("unexpected token: '#{tk2.inspect}'") if $DEBBUG
 	    break
 	  end
 	end
