@@ -1,12 +1,12 @@
 ;;;
 ;;;  ruby-mode.el -
 ;;;
-;;;  $Author: knu $
-;;;  $Date: 2008-05-19 00:02:36 +0900 (Mon, 19 May 2008) $
+;;;  $Author: matz $
+;;;  $Date: 2005/06/12 16:58:40 $
 ;;;  created at: Fri Feb  4 14:49:13 JST 1994
 ;;;
 
-(defconst ruby-mode-revision "$Revision: 16458 $")
+(defconst ruby-mode-revision "$Revision: 1.74.2.11 $")
 
 (defconst ruby-mode-version
   (progn
@@ -18,7 +18,7 @@
   )
 
 (defconst ruby-non-block-do-re
-  "\\(while\\|until\\|for\\|rescue\\)\\>[^_]"
+  "\\(while\\|until\\|for\\|rescue\\)\\>"
   )
 
 (defconst ruby-indent-beg-re
@@ -45,7 +45,7 @@
   (concat ruby-modifier-beg-re "\\|" ruby-block-op-re)
   )
 
-(defconst ruby-block-end-re "\\<end\\>")
+(defconst ruby-block-end-re "end")
 
 (defconst ruby-here-doc-beg-re
   "<<\\(-\\)?\\(\\([a-zA-Z0-9_]+\\)\\|[\"]\\([^\"]+\\)[\"]\\|[']\\([^']+\\)[']\\)")
@@ -61,13 +61,13 @@
 (defconst ruby-delimiter
   (concat "[?$/%(){}#\"'`.:]\\|<<\\|\\[\\|\\]\\|\\<\\("
 	  ruby-block-beg-re
-	  "\\)\\>\\|" ruby-block-end-re
-	  "\\|^=begin\\|" ruby-here-doc-beg-re)
+	  "\\|" ruby-block-end-re
+	  "\\)\\>\\|^=begin\\|" ruby-here-doc-beg-re)
   )
 
 (defconst ruby-negative
-  (concat "^[ \t]*\\(\\(" ruby-block-mid-re "\\)\\>\\|"
-	    ruby-block-end-re "\\|}\\|\\]\\)")
+  (concat "^[ \t]*\\(\\(" ruby-block-mid-re "\\)\\>\\|\\("
+	    ruby-block-end-re "\\)\\>\\|}\\|\\]\\)")
   )
 
 (defconst ruby-operator-chars "-,.+*/%&|^~=<>:")
@@ -152,7 +152,7 @@
 Also ignores spaces after parenthesis when 'space."
   :group 'ruby)
 
-(defcustom ruby-deep-indent-paren '(?\( ?\[ ?\] t)
+(defcustom ruby-deep-indent-paren '(?\( t)
   "*Deep indent lists in parenthesis when non-nil. t means continuous line.
 Also ignores spaces after parenthesis when 'space."
   :group 'ruby)
@@ -255,12 +255,7 @@ The variable ruby-indent-level controls the amount of indentation.
   (make-local-variable 'add-log-current-defun-function)
   (setq add-log-current-defun-function 'ruby-add-log-current-method)
 
-  (set (make-local-variable 'font-lock-defaults) '((ruby-font-lock-keywords) nil nil))
-  (set (make-local-variable 'font-lock-keywords) ruby-font-lock-keywords)
-  (set (make-local-variable 'font-lock-syntax-table) ruby-font-lock-syntax-table)
-  (set (make-local-variable 'font-lock-syntactic-keywords) ruby-font-lock-syntactic-keywords)
-
-  (run-mode-hooks 'ruby-mode-hook))
+  (run-hooks 'ruby-mode-hook))
 
 (defun ruby-current-indentation ()
   (save-excursion
@@ -317,13 +312,14 @@ The variable ruby-indent-level controls the amount of indentation.
 	       (or (eq (char-syntax (char-before (point))) ?w)
 		   (ruby-special-char-p))))
 	nil)
-       ((and (eq option 'heredoc) (< space 0)) t)
-       ((or (looking-at ruby-operator-re)
+       ((or (save-excursion (goto-char start) (looking-at ruby-operator-re))
 	    (looking-at "[\\[({,;]")
-	    (and (looking-at "[!?]")
+	    (and (or (not (eq option 'heredoc))
+		     (< space 0))
+		 (looking-at "[!?]")
 		 (or (not (eq option 'modifier))
 		     (bolp)
-		     (save-excursion (forward-char -1) (looking-at "\\Sw$"))))
+		     (save-excursion (forward-char -1) (looking-at "\\Sw"))))
 	    (and (looking-at ruby-symbol-re)
 		 (skip-chars-backward ruby-symbol-chars)
 		 (cond
@@ -331,7 +327,7 @@ The variable ruby-indent-level controls the amount of indentation.
 					   "|" ruby-block-op-re
 					   "|" ruby-block-mid-re "\\)\\>")))
 		   (goto-char (match-end 0))
-		   (not (looking-at "\\s_")))
+                  (not (looking-at "\\s_")))
 		  ((eq option 'expr-qstr)
 		   (looking-at "[a-zA-Z][a-zA-z0-9_]* +%[^ \t]"))
 		  ((eq option 'expr-re)
@@ -388,8 +384,6 @@ The variable ruby-indent-level controls the amount of indentation.
 	 (t
 	  (setq in-string (point))
 	  (goto-char end))))
-       ((looking-at "/=") 
-	(goto-char pnt))
        ((looking-at "/")
 	(cond
 	 ((and (not (eobp)) (ruby-expr-beg 'expr-re))
@@ -459,7 +453,7 @@ The variable ruby-indent-level controls the amount of indentation.
 	  (setq depth (1- depth)))
 	(setq nest (cdr nest))
 	(goto-char pnt))
-       ((looking-at ruby-block-end-re)
+       ((looking-at (concat "\\<\\(" ruby-block-end-re "\\)\\>"))
 	(if (or (and (not (bolp))
 		     (progn
 		       (forward-char -1)
@@ -511,9 +505,8 @@ The variable ruby-indent-level controls the amount of indentation.
 	 (setq nest (cons (cons nil pnt) nest))
 	 (setq depth (1+ depth)))
 	(goto-char pnt))
-       ((looking-at ":\\(['\"]\\)")
-	(goto-char (match-beginning 1))
-	(ruby-forward-string (buffer-substring (match-beginning 1) (match-end 1)) end))
+       ((looking-at ":\\(['\"]\\)\\(\\\\.\\|[^\\\\]\\)*\\1")
+	(goto-char (match-end 0)))
        ((looking-at ":\\([-,.+*/%&|^~<>]=?\\|===?\\|<=>\\)")
 	(goto-char (match-end 0)))
        ((looking-at ":\\([a-zA-Z_][a-zA-Z_0-9]*[!?=]?\\)?")
@@ -531,7 +524,7 @@ The variable ruby-indent-level controls the amount of indentation.
        ((looking-at "<<")
 	(cond
 	 ((and (ruby-expr-beg 'heredoc)
-	       (looking-at "<<\\(-\\)?\\(\\([\"'`]\\)\\([^\n]+?\\)\\3\\|\\(?:\\sw\\|\\s_\\)+\\)"))
+	       (looking-at "<<\\(-\\)?\\(\\([\"'`]\\)\\([^\n]+?\\)\\3\\|\\sw+\\)"))
 	  (setq re (regexp-quote (or (match-string 4) (match-string 2))))
 	  (if (match-beginning 1) (setq re (concat "\\s *" re)))
 	  (let* ((id-end (goto-char (match-end 0)))
@@ -625,11 +618,7 @@ The variable ruby-indent-level controls the amount of indentation.
 			   (t (setq indent (ruby-indent-size (1- indent) 1))))))
 	    (if (nth 3 state) (goto-char (nth 3 state))
 	      (goto-char parse-start) (back-to-indentation))
-	    (setq indent (ruby-indent-size (current-column) (nth 2 state))))
-	  (and (eq (car (nth 1 state)) paren)
-	       (ruby-deep-indent-paren-p (matching-paren paren))
-	       (search-backward (char-to-string paren))
-	       (setq indent (current-column)))))
+	    (setq indent (ruby-indent-size (current-column) (nth 2 state))))))
        ((and (nth 2 state) (> (nth 2 state) 0)) ; in nest
 	(if (null (cdr (nth 1 state)))
 	    (error "invalid nest"))
@@ -738,7 +727,6 @@ The variable ruby-indent-level controls the amount of indentation.
 		     (goto-char (or begin parse-start))
 		     (skip-syntax-forward " ")
 		     (current-column)))
-		  ((car (nth 1 state)) indent)
 		  (t
 		   (+ indent ruby-indent-level))))))))
       indent)))
@@ -788,8 +776,7 @@ An end of a defun is found by moving forward from the beginning of one."
 (defun ruby-move-to-block (n)
   (let (start pos done down)
     (setq start (ruby-calculate-indent))
-    (setq down (looking-at (if (< n 0) ruby-block-end-re
-			     (concat "\\<\\(" ruby-block-beg-re "\\)\\>"))))
+    (setq down (looking-at (concat "\\<\\(" (if (< n 0) ruby-block-end-re ruby-block-beg-re) "\\)\\>")))
     (while (and (not done) (not (if (< n 0) (bobp) (eobp))))
       (forward-line n)
       (cond
@@ -900,7 +887,7 @@ An end of a defun is found by moving forward from the beginning of one."
 				   (?:
 				    (forward-char -1)
 				    (eq (char-before) :)))))
-		   (if (looking-at ruby-block-end-re)
+		   (if (looking-at (concat "\\<\\(" ruby-block-end-re "\\)\\>"))
 		       (ruby-beginning-of-block))
 		   nil))
 	    (setq i (1- i)))
@@ -1019,19 +1006,30 @@ balanced expression is found."
 	  ;; ?' ?" ?` are ascii codes
 	  ("\\(^\\|[^\\\\]\\)\\(\\\\\\\\\\)*[?$]\\([#\"'`]\\)" 3 (1 . nil))
 	  ;; regexps
-	  ("\\(^\\|[=(,~?:;<>]\\|\\(^\\|\\s \\)\\(if\\|elsif\\|unless\\|while\\|until\\|when\\|and\\|or\\|&&\\|||\\)\\|g?sub!?\\|scan\\|split!?\\)\\s *\\(/\\)[^/\n\\\\]*\\(\\\\.[^/\n\\\\]*\\)*\\(/\\)"
+	  ("\\(^\\|[=(,~?:;]\\|\\(^\\|\\s \\)\\(if\\|elsif\\|unless\\|while\\|until\\|when\\|and\\|or\\|&&\\|||\\)\\|g?sub!?\\|scan\\|split!?\\)\\s *\\(/\\)[^/\n\\\\]*\\(\\\\.[^/\n\\\\]*\\)*\\(/\\)"
 	   (4 (7 . ?/))
 	   (6 (7 . ?/)))
 	  ("^\\(=\\)begin\\(\\s \\|$\\)" 1 (7 . nil))
 	  ("^\\(=\\)end\\(\\s \\|$\\)" 1 (7 . nil))))
 
-  (if (featurep 'xemacs)
-      (put 'ruby-mode 'font-lock-defaults
-	   '((ruby-font-lock-keywords)
-	     nil nil nil
-	     beginning-of-line
-	     (font-lock-syntactic-keywords
-	      . ruby-font-lock-syntactic-keywords))))
+  (cond ((featurep 'xemacs)
+	 (put 'ruby-mode 'font-lock-defaults
+	      '((ruby-font-lock-keywords)
+		nil nil nil
+		beginning-of-line
+		(font-lock-syntactic-keywords
+		 . ruby-font-lock-syntactic-keywords))))
+	(t
+	 (add-hook 'ruby-mode-hook
+	    '(lambda ()
+	       (make-local-variable 'font-lock-defaults)
+	       (make-local-variable 'font-lock-keywords)
+	       (make-local-variable 'font-lock-syntax-table)
+	       (make-local-variable 'font-lock-syntactic-keywords)
+	       (setq font-lock-defaults '((ruby-font-lock-keywords) nil nil))
+	       (setq font-lock-keywords ruby-font-lock-keywords)
+	       (setq font-lock-syntax-table ruby-font-lock-syntax-table)
+	       (setq font-lock-syntactic-keywords ruby-font-lock-syntactic-keywords)))))
 
   (defun ruby-font-lock-docs (limit)
     (if (re-search-forward "^=begin\\(\\s \\|$\\)" limit t)
