@@ -3,8 +3,8 @@
   missing.h - prototype for *.c in ./missing, and
   	      for missing timeval struct
 
-  $Author: knu $
-  $Date: 2008-06-06 19:39:57 +0900 (Fri, 06 Jun 2008) $
+  $Author: matz $
+  $Date: 2005/06/12 16:58:41 $
   created at: Sat May 11 23:46:03 JST 2002
 
 ************************************************/
@@ -32,17 +32,13 @@ extern double atanh _((double));
 #endif
 
 #ifndef HAVE_CRYPT
-extern char *crypt _((const char *, const char *));
+extern char *crypt _((char *, char *));
 #endif
 
 #ifndef HAVE_DUP2
 extern int dup2 _((int, int));
 #endif
 
-#ifndef HAVE_EACCESS
-extern int eaccess _((const char*, int));
-#endif
-
 #ifndef HAVE_FINITE
 extern int finite _((double));
 #endif
