@@ -2,8 +2,8 @@
 
   object.c -
 
-  $Author: shyouhei $
-  $Date: 2008-07-03 20:14:50 +0900 (Thu, 03 Jul 2008) $
+  $Author: matz $
+  $Date: 2005/09/08 05:59:33 $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -98,6 +98,46 @@ rb_obj_equal(obj1, obj2)
     return Qfalse;
 }
 
+
+/*
+ *  Document-method: __id__
+ *  Document-method: object_id
+ *
+ *  call-seq:
+ *     obj.__id__       => fixnum
+ *     obj.object_id    => fixnum
+ *  
+ *  Returns an integer identifier for <i>obj</i>. The same number will
+ *  be returned on all calls to <code>id</code> for a given object, and
+ *  no two active objects will share an id.
+ *  <code>Object#object_id</code> is a different concept from the
+ *  <code>:name</code> notation, which returns the symbol id of
+ *  <code>name</code>. Replaces the deprecated <code>Object#id</code>.
+ */
+
+
+
+/*
+ *  call-seq:
+ *     obj.hash    => fixnum
+ *  
+ *  Generates a <code>Fixnum</code> hash value for this object. This
+ *  function must have the property that <code>a.eql?(b)</code> implies
+ *  <code>a.hash == b.hash</code>. The hash value is used by class
+ *  <code>Hash</code>. Any hash value that exceeds the capacity of a
+ *  <code>Fixnum</code> will be truncated before being used.
+ */
+
+VALUE
+rb_obj_id(obj)
+    VALUE obj;
+{
+    if (SPECIAL_CONST_P(obj)) {
+	return LONG2NUM((long)obj);
+    }
+    return (VALUE)((long)obj|FIXNUM_FLAG);
+}
+
 /*
  *  call-seq:
  *     obj.id    => fixnum
@@ -263,7 +303,6 @@ rb_obj_dup(obj)
     return dup;
 }
 
-/* :nodoc: */
 VALUE
 rb_obj_init_copy(obj, orig)
     VALUE obj, orig;
@@ -314,7 +353,7 @@ VALUE
 rb_any_to_s(obj)
     VALUE obj;
 {
-    const char *cname = rb_obj_classname(obj);
+    char *cname = rb_obj_classname(obj);
     size_t len;
     VALUE str;
 
@@ -341,7 +380,7 @@ inspect_i(id, value, str)
     VALUE str;
 {
     VALUE str2;
-    const char *ivname;
+    char *ivname;
 
     /* need not to show internal data */
     if (CLASS_OF(value) == 0) return ST_CONTINUE;
@@ -397,8 +436,9 @@ rb_obj_inspect(obj)
 	&& ROBJECT(obj)->iv_tbl->num_entries > 0) {
 	VALUE str;
 	size_t len;
-	const char *c = rb_obj_classname(obj);
+	char *c;
 
+	c = rb_obj_classname(obj);
 	if (rb_inspecting_p(obj)) {
 	    len = strlen(c)+10+16+1;
 	    str = rb_str_new(0, len); /* 10:tags 16:addr 1:nul */
@@ -493,63 +533,6 @@ rb_obj_is_kind_of(obj, c)
 }
 
 
-/*
- *  call-seq:
- *     obj.tap{|x|...}    => obj
- *  
- *  Yields <code>x</code> to the block, and then returns <code>x</code>.
- *  The primary purpose of this method is to "tap into" a method chain,
- *  in order to perform operations on intermediate results within the chain.
- *
- *	(1..10).tap {
- *	  |x| puts "original: #{x.inspect}"
- *	}.to_a.tap {
- *	  |x| puts "array: #{x.inspect}"
- *	}.select {|x| x%2==0}.tap {
- *	  |x| puts "evens: #{x.inspect}"
- *	}.map {|x| x*x}.tap {
- *	  |x| puts "squares: #{x.inspect}"
- *	}
- *
- */
-
-VALUE
-rb_obj_tap(obj)
-    VALUE obj;
-{
-    rb_yield(obj);
-    return obj;
-}
-
-
-/*
- * Document-method: inherited
- *
- * call-seq:
- *    inherited(subclass)
- *
- * Callback invoked whenever a subclass of the current class is created.
- *
- * Example:
- *
- *    class Foo
- *       def self.inherited(subclass)
- *          puts "New subclass: #{subclass}"
- *       end
- *    end
- *
- *    class Bar < Foo
- *    end
- *
- *    class Baz < Bar
- *    end
- *
- * produces:
- *
- *    New subclass: Bar
- *    New subclass: Baz
- */
-
 /*
  * Document-method: singleton_method_added
  *
@@ -661,6 +644,7 @@ rb_obj_dummy()
     return Qnil;
 }
 
+
 /*
  *  call-seq:
  *     obj.tainted?    => true or false
@@ -867,6 +851,28 @@ nil_inspect(obj)
     return rb_str_new2("nil");
 }
 
+#ifdef NIL_PLUS
+static VALUE
+nil_plus(x, y)
+    VALUE x, y;
+{
+    switch (TYPE(y)) {
+      case T_NIL:
+      case T_FIXNUM:
+      case T_FLOAT:
+      case T_BIGNUM:
+      case T_STRING:
+      case T_ARRAY:
+	return y;
+      default:
+	rb_raise(rb_eTypeError, "tried to add %s(%s) to nil",
+		 RSTRING(rb_inspect(y))->ptr,
+		 rb_obj_classname(y));
+    }
+    /* not reached */
+}
+#endif
+
 static VALUE
 main_to_s(obj)
     VALUE obj;
@@ -1160,14 +1166,14 @@ sym_inspect(sym)
     VALUE sym;
 {
     VALUE str;
-    const char *name;
+    char *name;
     ID id = SYM2ID(sym);
 
     name = rb_id2name(id);
     str = rb_str_new(0, strlen(name)+1);
     RSTRING(str)->ptr[0] = ':';
     strcpy(RSTRING(str)->ptr+1, name);
-    if (!rb_symname_p(name)) {
+    if (rb_is_junk_id(id)) {
 	str = rb_str_dump(str);
 	strncpy(RSTRING(str)->ptr, ":\"", 2);
     }
@@ -1211,37 +1217,6 @@ sym_to_sym(sym)
 }
 
 
-static VALUE
-sym_call(args, mid)
-    VALUE args, mid;
-{
-    VALUE obj;
-
-    if (RARRAY(args)->len < 1) {
-	rb_raise(rb_eArgError, "no receiver given");
-    }
-    obj = rb_ary_shift(args);
-    return rb_apply(obj, (ID)mid, args);
-}
-
-VALUE rb_proc_new _((VALUE (*)(ANYARGS/* VALUE yieldarg[, VALUE procarg] */), VALUE));
-
-/*
- * call-seq:
- *   sym.to_proc
- *
- * Returns a _Proc_ object which respond to the given method by _sym_.
- *
- *   (1..3).collect(&:to_s)  #=> ["1", "2", "3"]
- */
-
-static VALUE
-sym_to_proc(VALUE sym)
-{
-    return rb_proc_new(sym_call, (VALUE)SYM2ID(sym));
-}
-
-
 /***********************************************************************
  *
  * Document-class: Module
@@ -1573,21 +1548,8 @@ rb_class_initialize(argc, argv, klass)
  *  call-seq:
  *     class.allocate()   =>   obj
  *  
- *  Allocates space for a new object of <i>class</i>'s class and does not
- *  call initialize on the new instance. The returned object must be an
- *  instance of <i>class</i>.
- *  
- *      klass = Class.new do
- *        def initialize(*args)
- *          @initialized = true
- *        end
- *      
- *        def initialized?
- *          @initialized || false
- *        end
- *      end
- *      
- *      klass.allocate.initialized? #=> false
+ *  Allocates space for a new object of <i>class</i>'s class. The
+ *  returned object must be an instance of <i>class</i>.
  *     
  */
 
@@ -1683,9 +1645,13 @@ static ID
 str_to_id(str)
     VALUE str;
 {
-    VALUE sym = rb_str_intern(str);
-
-    return SYM2ID(sym);
+    if (!RSTRING(str)->ptr || RSTRING(str)->len == 0) {
+	rb_raise(rb_eArgError, "empty symbol string");
+    }
+    if (RSTRING(str)->len != strlen(RSTRING(str)->ptr)) {
+	rb_warn("Symbols should not contain NUL (\\0)");
+    }
+    return rb_intern(RSTRING(str)->ptr);
 }
 
 ID
@@ -1916,6 +1882,7 @@ rb_mod_const_defined(mod, name)
  *     k.methods.length   #=> 42
  */
 
+
 static VALUE
 rb_obj_methods(argc, argv, obj)
     int argc;
@@ -2016,12 +1983,10 @@ rb_obj_public_methods(argc, argv, obj)
 /*
  *  call-seq:
  *     obj.instance_variable_get(symbol)    => obj
- *
- *  Returns the value of the given instance variable, or nil if the
- *  instance variable is not set. The <code>@</code> part of the
- *  variable name should be included for regular instance
- *  variables. Throws a <code>NameError</code> exception if the
- *  supplied symbol is not valid as an instance variable name.
+ *  
+ *  Returns the value of the given instance variable (or throws a
+ *  <code>NameError</code> exception). The <code>@</code> part of the
+ *  variable name should be included for regular instance variables
  *     
  *     class Fred
  *       def initialize(p1, p2)
@@ -2045,6 +2010,7 @@ rb_obj_ivar_get(obj, iv)
     return rb_ivar_get(obj, id);
 }
 
+
 /*
  *  call-seq:
  *     obj.instance_variable_set(symbol, obj)    => obj
@@ -2077,36 +2043,6 @@ rb_obj_ivar_set(obj, iv, val)
     return rb_ivar_set(obj, id, val);
 }
 
-/*
- *  call-seq:
- *     obj.instance_variable_defined?(symbol)    => true or false
- *
- *  Returns <code>true</code> if the given instance variable is
- *  defined in <i>obj</i>.
- *
- *     class Fred
- *       def initialize(p1, p2)
- *         @a, @b = p1, p2
- *       end
- *     end
- *     fred = Fred.new('cat', 99)
- *     fred.instance_variable_defined?(:@a)    #=> true
- *     fred.instance_variable_defined?("@b")   #=> true
- *     fred.instance_variable_defined?("@c")   #=> false
- */
-
-static VALUE
-rb_obj_ivar_defined(obj, iv)
-    VALUE obj, iv;
-{
-    ID id = rb_to_id(iv);
-
-    if (!rb_is_instance_id(id)) {
-	rb_name_error(id, "`%s' is not allowed as an instance variable name", rb_id2name(id));
-    }
-    return rb_ivar_defined(obj, id);
-}
-
 /*
  *  call-seq:
  *     mod.class_variable_get(symbol)    => obj
@@ -2131,11 +2067,12 @@ rb_mod_cvar_get(obj, iv)
     ID id = rb_to_id(iv);
 
     if (!rb_is_class_id(id)) {
-	rb_name_error(id, "`%s' is not allowed as a class variable name", rb_id2name(id));
+	rb_name_error(id, "`%s' is not allowed as an class variable name", rb_id2name(id));
     }
     return rb_cvar_get(obj, id);
 }
 
+
 /*
  *  call-seq:
  *     obj.class_variable_set(symbol, obj)    => obj
@@ -2164,38 +2101,12 @@ rb_mod_cvar_set(obj, iv, val)
     ID id = rb_to_id(iv);
 
     if (!rb_is_class_id(id)) {
-	rb_name_error(id, "`%s' is not allowed as a class variable name", rb_id2name(id));
+	rb_name_error(id, "`%s' is not allowed as an class variable name", rb_id2name(id));
     }
     rb_cvar_set(obj, id, val, Qfalse);
     return val;
 }
 
-/*
- *  call-seq:
- *     obj.class_variable_defined?(symbol)    => true or false
- *
- *  Returns <code>true</code> if the given class variable is defined
- *  in <i>obj</i>.
- *
- *     class Fred
- *       @@foo = 99
- *     end
- *     Fred.class_variable_defined?(:@@foo)    #=> true
- *     Fred.class_variable_defined?(:@@bar)    #=> false
- */
-
-static VALUE
-rb_mod_cvar_defined(obj, iv)
-    VALUE obj, iv;
-{
-    ID id = rb_to_id(iv);
-
-    if (!rb_is_class_id(id)) {
-	rb_name_error(id, "`%s' is not allowed as a class variable name", rb_id2name(id));
-    }
-    return rb_cvar_defined(obj, id);
-}
-
 static VALUE
 convert_type(val, tname, method, raise)
     VALUE val;
@@ -2271,19 +2182,6 @@ rb_to_integer(val, method)
     return v;
 }
 
-VALUE
-rb_check_to_integer(VALUE val, const char *method)
-{
-    VALUE v;
-
-    if (FIXNUM_P(val)) return val;
-    v = convert_type(val, "Integer", method, Qfalse);
-    if (!rb_obj_is_kind_of(v, rb_cInteger)) {
-	return Qnil;
-    }
-    return v;
-}
-
 VALUE
 rb_to_int(val)
     VALUE val;
@@ -2354,9 +2252,6 @@ rb_cstr_to_dbl(p, badcheck)
     const char *q;
     char *end;
     double d;
-    const char *ellipsis = "";
-    int w;
-#define OutOfRange() (((w = end - p) > 20) ? (w = 20, ellipsis = "...") : (ellipsis = ""))
 
     if (!p) return 0.0;
     q = p;
@@ -2366,11 +2261,9 @@ rb_cstr_to_dbl(p, badcheck)
     else {
 	while (ISSPACE(*p) || *p == '_') p++;
     }
-    errno = 0;
     d = strtod(p, &end);
     if (errno == ERANGE) {
-	OutOfRange();
-	rb_warn("Float %.*s%s out of range", w, p, ellipsis);
+	rb_warn("Float %*s out of range", end-p, p);
 	errno = 0;
     }
     if (p == end) {
@@ -2404,20 +2297,18 @@ rb_cstr_to_dbl(p, badcheck)
 	p = buf;
 	d = strtod(p, &end);
 	if (errno == ERANGE) {
-	    OutOfRange();
-	    rb_warn("Float %.*s%s out of range", w, p, ellipsis);
+	    rb_warn("Float %*s out of range", end-p, p);
 	    errno = 0;
 	}
 	if (badcheck) {
-	    if (!end || p == end) goto bad;
+	    if (p == end) goto bad;
 	    while (*end && ISSPACE(*end)) end++;
 	    if (*end) goto bad;
 	}
     }
     if (errno == ERANGE) {
 	errno = 0;
-	OutOfRange();
-	rb_raise(rb_eArgError, "Float %.*s%s out of range", w, q, ellipsis);
+	rb_raise(rb_eArgError, "Float %s out of range", q);
     }
     return d;
 }
@@ -2722,7 +2613,10 @@ Init_Object()
 
     rb_define_method(rb_mKernel, "eql?", rb_obj_equal, 1);
 
+    rb_define_method(rb_mKernel, "hash", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "id", rb_obj_id_obsolete, 0);
+    rb_define_method(rb_mKernel, "__id__", rb_obj_id, 0);
+    rb_define_method(rb_mKernel, "object_id", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "type", rb_obj_type, 0);
     rb_define_method(rb_mKernel, "class", rb_obj_class, 0);
 
@@ -2750,14 +2644,12 @@ Init_Object()
 		     rb_obj_instance_variables, 0); /* in variable.c */
     rb_define_method(rb_mKernel, "instance_variable_get", rb_obj_ivar_get, 1);
     rb_define_method(rb_mKernel, "instance_variable_set", rb_obj_ivar_set, 2);
-    rb_define_method(rb_mKernel, "instance_variable_defined?", rb_obj_ivar_defined, 1);
     rb_define_private_method(rb_mKernel, "remove_instance_variable",
 			     rb_obj_remove_instance_variable, 1); /* in variable.c */
 
     rb_define_method(rb_mKernel, "instance_of?", rb_obj_is_instance_of, 1);
     rb_define_method(rb_mKernel, "kind_of?", rb_obj_is_kind_of, 1);
     rb_define_method(rb_mKernel, "is_a?", rb_obj_is_kind_of, 1);
-    rb_define_method(rb_mKernel, "tap", rb_obj_tap, 0);
 
     rb_define_private_method(rb_mKernel, "singleton_method_added", rb_obj_dummy, 1);
     rb_define_private_method(rb_mKernel, "singleton_method_removed", rb_obj_dummy, 1);
@@ -2799,7 +2691,6 @@ Init_Object()
     rb_define_method(rb_cSymbol, "to_s", sym_to_s, 0);
     rb_define_method(rb_cSymbol, "id2name", sym_to_s, 0);
     rb_define_method(rb_cSymbol, "to_sym", sym_to_sym, 0);
-    rb_define_method(rb_cSymbol, "to_proc", sym_to_proc, 0);
     rb_define_method(rb_cSymbol, "===", rb_obj_equal, 1); 
 
     rb_define_method(rb_cModule, "freeze", rb_mod_freeze, 0);
@@ -2810,7 +2701,7 @@ Init_Object()
     rb_define_method(rb_cModule, "<=", rb_class_inherited_p, 1);
     rb_define_method(rb_cModule, ">",  rb_mod_gt, 1);
     rb_define_method(rb_cModule, ">=", rb_mod_ge, 1);
-    rb_define_method(rb_cModule, "initialize_copy", rb_mod_init_copy, 1); /* in class.c */
+    rb_define_method(rb_cModule, "initialize_copy", rb_mod_init_copy, 1);
     rb_define_method(rb_cModule, "to_s", rb_mod_to_s, 0);
     rb_define_method(rb_cModule, "included_modules", 
 		     rb_mod_included_modules, 0); /* in class.c */
@@ -2834,7 +2725,6 @@ Init_Object()
     rb_define_method(rb_cModule, "private_instance_methods", 
 		     rb_class_private_instance_methods, -1);   /* in class.c */
 
-    rb_define_method(rb_cModule, "class_variable_defined?", rb_mod_cvar_defined, 1);
     rb_define_method(rb_cModule, "constants", rb_mod_constants, 0); /* in variable.c */
     rb_define_method(rb_cModule, "const_get", rb_mod_const_get, 1);
     rb_define_method(rb_cModule, "const_set", rb_mod_const_set, 2);
@@ -2853,7 +2743,7 @@ Init_Object()
     rb_define_method(rb_cClass, "allocate", rb_obj_alloc, 0);
     rb_define_method(rb_cClass, "new", rb_class_new_instance, -1);
     rb_define_method(rb_cClass, "initialize", rb_class_initialize, -1);
-    rb_define_method(rb_cClass, "initialize_copy", rb_class_init_copy, 1); /* in class.c */
+    rb_define_method(rb_cClass, "initialize_copy", rb_class_init_copy, 1);
     rb_define_method(rb_cClass, "superclass", rb_class_superclass, 0);
     rb_define_alloc_func(rb_cClass, rb_class_s_alloc);
     rb_undef_method(rb_cClass, "extend_object");
@@ -2862,8 +2752,8 @@ Init_Object()
     rb_cData = rb_define_class("Data", rb_cObject);
     rb_undef_alloc_func(rb_cData);
 
-    rb_global_variable(&ruby_top_self);
     ruby_top_self = rb_obj_alloc(rb_cObject);
+    rb_global_variable(&ruby_top_self);
     rb_define_singleton_method(ruby_top_self, "to_s", main_to_s, 0);
 
     rb_cTrueClass = rb_define_class("TrueClass", rb_cObject);
