@@ -2,7 +2,7 @@
 
   re.c -
 
-  $Author: shyouhei $
+  $Author: matz $
   created at: Mon Aug  9 18:24:49 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -13,7 +13,7 @@
 #include "re.h"
 #include <ctype.h>
 
-VALUE rb_eRegexpError;
+static VALUE rb_eRegexpError;
 
 #define BEG(no) regs->beg[no]
 #define END(no) regs->end[no]
@@ -70,15 +70,14 @@ static const char casetable[] = {
 #endif
 
 int
-rb_memcicmp(x, y, len)
-    const void *x, *y;
+rb_memcicmp(p1, p2, len)
+    char *p1, *p2;
     long len;
 {
-    const unsigned char *p1 = x, *p2 = y;
     int tmp;
 
     while (len--) {
-	if ((tmp = casetable[(unsigned)*p1++] - casetable[(unsigned)*p2++]) != 0)
+	if (tmp = casetable[(unsigned)*p1++] - casetable[(unsigned)*p2++])
 	    return tmp;
     }
     return 0;
@@ -86,7 +85,7 @@ rb_memcicmp(x, y, len)
 
 int
 rb_memcmp(p1, p2, len)
-    const void *p1, *p2;
+    char *p1, *p2;
     long len;
 {
     if (!ruby_ignorecase) {
@@ -97,16 +96,16 @@ rb_memcmp(p1, p2, len)
 
 long
 rb_memsearch(x0, m, y0, n)
-    const void *x0, *y0;
+    char *x0, *y0;
     long m, n;
 {
-    const unsigned char *x = (unsigned char *)x0, *y = (unsigned char *)y0;
-    const unsigned char *s, *e;
+    unsigned char *x = (unsigned char *)x0, *y = (unsigned char *)y0;
+    unsigned char *s, *e;
     long i;
     int d;
     unsigned long hx, hy;
 
-#define KR_REHASH(a, b, h) (((h) << 1) - (((unsigned long)(a))<<d) + (b))
+#define KR_REHASH(a, b, h) (((h) << 1) - ((a)<<d) + (b))
 
     if (m > n) return -1;
     s = y; e = s + n - m;
@@ -152,7 +151,6 @@ rb_memsearch(x0, m, y0, n)
     return s-y;
 }
 
-#define REG_LITERAL FL_USER5
 #define REG_CASESTATE  FL_USER0
 #define KCODE_NONE  0
 #define KCODE_EUC   FL_USER1
@@ -200,8 +198,8 @@ kcode_none(re)
 
 static int curr_kcode;
 
-void
-rb_kcode_set_option(re)
+static void
+kcode_set_option(re)
     VALUE re;
 {
     if (!FL_TEST(re, KCODE_FIXED)) return;
@@ -224,8 +222,8 @@ rb_kcode_set_option(re)
     }
 }	  
 
-void
-rb_kcode_reset_option()
+static void
+kcode_reset_option()
 {
     if (reg_kcode == curr_kcode) return;
     switch (reg_kcode) {
@@ -253,9 +251,9 @@ rb_reg_mbclen2(c, re)
 
     if (!FL_TEST(re, KCODE_FIXED))
 	return mbclen(c);
-    rb_kcode_set_option(re);
+    kcode_set_option(re);
     len = mbclen(c);
-    rb_kcode_reset_option();
+    kcode_reset_option();
     return len;
 }
 
@@ -486,11 +484,11 @@ rb_reg_to_s(re)
 	}
 	if (*ptr == ':' && ptr[len-1] == ')') {
 	    Regexp *rp;
-	    rb_kcode_set_option(re);
+	    kcode_set_option(re);
 	    rp = ALLOC(Regexp);
 	    MEMZERO((char *)rp, Regexp, 1);
 	    err = re_compile_pattern(++ptr, len -= 2, rp) != 0;
-	    rb_kcode_reset_option();
+	    kcode_reset_option();
 	    re_free_pattern(rp);
 	}
 	if (err) {
@@ -595,7 +593,7 @@ static VALUE
 rb_reg_kcode_m(re)
     VALUE re;
 {
-    const char *kcode;
+    char *kcode;
 
     if (FL_TEST(re, KCODE_FIXED)) {
 	switch (RBASIC(re)->flags & KCODE_MASK) {
@@ -623,7 +621,7 @@ make_regexp(s, len, flags)
     int flags;
 {
     Regexp *rp;
-    const char *err;
+    char *err;
 
     /* Handle escaped characters first. */
 
@@ -664,7 +662,7 @@ make_regexp(s, len, flags)
  *
  */
 
-VALUE rb_cMatch;
+static VALUE rb_cMatch;
 
 static VALUE match_alloc _((VALUE));
 static VALUE
@@ -682,15 +680,6 @@ match_alloc(klass)
     return (VALUE)match;
 }
 
-static void
-match_check(VALUE match)
-{
-    if (!RMATCH(match)->str) {
-	rb_raise(rb_eTypeError, "uninitialized Match");
-    }
-}
-
-/* :nodoc: */
 static VALUE
 match_init_copy(obj, orig)
     VALUE obj, orig;
@@ -725,7 +714,6 @@ static VALUE
 match_size(match)
     VALUE match;
 {
-    match_check(match);
     return INT2FIX(RMATCH(match)->regs->num_regs);
 }
 
@@ -748,7 +736,6 @@ match_offset(match, n)
 {
     int i = NUM2INT(n);
 
-    match_check(match);
     if (i < 0 || RMATCH(match)->regs->num_regs <= i)
 	rb_raise(rb_eIndexError, "index %d out of matches", i);
 
@@ -778,7 +765,6 @@ match_begin(match, n)
 {
     int i = NUM2INT(n);
 
-    match_check(match);
     if (i < 0 || RMATCH(match)->regs->num_regs <= i)
 	rb_raise(rb_eIndexError, "index %d out of matches", i);
 
@@ -807,7 +793,6 @@ match_end(match, n)
 {
     int i = NUM2INT(n);
 
-    match_check(match);
     if (i < 0 || RMATCH(match)->regs->num_regs <= i)
 	rb_raise(rb_eIndexError, "index %d out of matches", i);
 
@@ -858,10 +843,10 @@ rb_reg_prepare_re(re)
     }
 
     if (need_recompile) {
-	const char *err;
+	char *err;
 
 	if (FL_TEST(re, KCODE_FIXED))
-	    rb_kcode_set_option(re);
+	    kcode_set_option(re);
 	rb_reg_check(re);
 	RREGEXP(re)->ptr->fastmap_accurate = 0;
 	err = re_compile_pattern(RREGEXP(re)->str, RREGEXP(re)->len, RREGEXP(re)->ptr);
@@ -882,9 +867,9 @@ rb_reg_adjust_startpos(re, str, pos, reverse)
     if (may_need_recompile) rb_reg_prepare_re(re);
 
     if (FL_TEST(re, KCODE_FIXED))
-	rb_kcode_set_option(re);
+	kcode_set_option(re);
     else if (reg_kcode != curr_kcode)
-	rb_kcode_reset_option();
+	kcode_reset_option();
 
     if (reverse) {
 	range = -pos;
@@ -904,7 +889,7 @@ rb_reg_search(re, str, pos, reverse)
 {
     long result;
     VALUE match;
-    struct re_registers regs;
+    static struct re_registers regs;
     long range;
 
     if (pos > RSTRING(str)->len || pos < 0) {
@@ -916,9 +901,9 @@ rb_reg_search(re, str, pos, reverse)
     if (may_need_recompile) rb_reg_prepare_re(re);
 
     if (FL_TEST(re, KCODE_FIXED))
-	rb_kcode_set_option(re);
+	kcode_set_option(re);
     else if (reg_kcode != curr_kcode)
-	rb_kcode_reset_option();
+	kcode_reset_option();
 
     if (reverse) {
 	range = -pos;
@@ -926,12 +911,11 @@ rb_reg_search(re, str, pos, reverse)
     else {
 	range = RSTRING(str)->len - pos;
     }
-    MEMZERO(&regs, struct re_registers, 1);
     result = re_search(RREGEXP(re)->ptr,RSTRING(str)->ptr,RSTRING(str)->len,
 		       pos, range, &regs);
 
     if (FL_TEST(re, KCODE_FIXED))
-	rb_kcode_reset_option();
+	kcode_reset_option();
 
     if (result == -2) {
 	rb_reg_raise(RREGEXP(re)->str, RREGEXP(re)->len,
@@ -939,7 +923,6 @@ rb_reg_search(re, str, pos, reverse)
     }
 
     if (result < 0) {
-	re_free_registers(&regs);
 	rb_backref_set(Qnil);
 	return result;
     }
@@ -956,7 +939,6 @@ rb_reg_search(re, str, pos, reverse)
     }
 
     re_copy_registers(RMATCH(match)->regs, &regs);
-    re_free_registers(&regs);
     RMATCH(match)->str = rb_str_new4(str);
     rb_backref_set(match);
 
@@ -971,7 +953,6 @@ rb_reg_nth_defined(nth, match)
     VALUE match;
 {
     if (NIL_P(match)) return Qnil;
-    match_check(match);
     if (nth >= RMATCH(match)->regs->num_regs) {
 	return Qnil;
     }
@@ -992,7 +973,6 @@ rb_reg_nth_match(nth, match)
     long start, end, len;
 
     if (NIL_P(match)) return Qnil;
-    match_check(match);
     if (nth >= RMATCH(match)->regs->num_regs) {
 	return Qnil;
     }
@@ -1035,7 +1015,6 @@ rb_reg_match_pre(match)
     VALUE str;
 
     if (NIL_P(match)) return Qnil;
-    match_check(match);
     if (RMATCH(match)->BEG(0) == -1) return Qnil;
     str = rb_str_substr(RMATCH(match)->str, 0, RMATCH(match)->BEG(0));
     if (OBJ_TAINTED(match)) OBJ_TAINT(str);
@@ -1062,7 +1041,6 @@ rb_reg_match_post(match)
     long pos;
 
     if (NIL_P(match)) return Qnil;
-    match_check(match);
     if (RMATCH(match)->BEG(0) == -1) return Qnil;
     str = RMATCH(match)->str;
     pos = RMATCH(match)->END(0);
@@ -1078,7 +1056,6 @@ rb_reg_match_last(match)
     int i;
 
     if (NIL_P(match)) return Qnil;
-    match_check(match);
     if (RMATCH(match)->BEG(0) == -1) return Qnil;
 
     for (i=RMATCH(match)->regs->num_regs-1; RMATCH(match)->BEG(i) == -1 && i > 0; i--)
@@ -1116,17 +1093,12 @@ match_array(match, start)
     VALUE match;
     int start;
 {
-    struct re_registers *regs;
-    VALUE ary;
-    VALUE target;
+    struct re_registers *regs = RMATCH(match)->regs;
+    VALUE ary = rb_ary_new2(regs->num_regs);
+    VALUE target = RMATCH(match)->str;
     int i;
     int taint = OBJ_TAINTED(match);
-
-    match_check(match);
-    regs = RMATCH(match)->regs;
-    ary = rb_ary_new2(regs->num_regs);
-    target = RMATCH(match)->str;
-
+    
     for (i=start; i<regs->num_regs; i++) {
 	if (regs->beg[i] == -1) {
 	    rb_ary_push(ary, Qnil);
@@ -1242,14 +1214,14 @@ match_entry(match, n)
 
 /*
  *  call-seq:
- *     mtch.values_at([index]*)   => array
+ *     mtch.select([index]*)   => array
  *  
  *  Uses each <i>index</i> to access the matching values, returning an array of
  *  the corresponding matches.
  *     
  *     m = /(.)(.)(\d+)(\d)/.match("THX1138: The Movie")
  *     m.to_a               #=> ["HX1138", "H", "X", "113", "8"]
- *     m.values_at(0, 2, -2)   #=> ["HX1138", "X", "113"]
+ *     m.select(0, 2, -2)   #=> ["HX1138", "X", "113"]
  */
 
 static VALUE
@@ -1258,20 +1230,20 @@ match_values_at(argc, argv, match)
     VALUE *argv;
     VALUE match;
 {
-    match_check(match);
     return rb_values_at(match, RMATCH(match)->regs->num_regs, argc, argv, match_entry);
 }
 
 
 /*
  *  call-seq:
- *     mtch.select{|obj| block}   => array
+ *     mtch.select([index]*)   => array
  *  
- *  Returns an array containing match strings for which <em>block</em>
- *  gives <code>true</code>.  MatchData#select will be removed from Ruby 1.9.
+ *  Uses each <i>index</i> to access the matching values, returning an
+ *  array of the corresponding matches.
  *     
  *     m = /(.)(.)(\d+)(\d)/.match("THX1138: The Movie")
- *     p m.select{|x| /X/ =~ x}   #=> ["HX1138", "X"]
+ *     m.to_a               #=> ["HX1138", "H", "X", "113", "8"]
+ *     m.select(0, 2, -2)   #=> ["HX1138", "X", "113"]
  */
 
 static VALUE
@@ -1284,16 +1256,12 @@ match_select(argc, argv, match)
 	rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)", argc);
     }
     else {
-	struct re_registers *regs;
-	VALUE target;
+	struct re_registers *regs = RMATCH(match)->regs;
+	VALUE target = RMATCH(match)->str;
 	VALUE result = rb_ary_new();
 	int i;
 	int taint = OBJ_TAINTED(match);
 
-	match_check(match);
-	regs = RMATCH(match)->regs;
-	target = RMATCH(match)->str;
-
 	for (i=0; i<regs->num_regs; i++) {
 	    VALUE str = rb_str_substr(target, regs->beg[i], regs->end[i]-regs->beg[i]);
 	    if (taint) OBJ_TAINT(str);
@@ -1306,7 +1274,6 @@ match_select(argc, argv, match)
 }
 
 
-
 /*
  *  call-seq:
  *     mtch.to_s   => str
@@ -1344,59 +1311,9 @@ static VALUE
 match_string(match)
     VALUE match;
 {
-    match_check(match);
     return RMATCH(match)->str;	/* str is frozen */
 }
 
-/*
- * call-seq:
- *    mtch.inspect   => str
- *
- * Returns a printable version of <i>mtch</i>.
- *
- *     puts /.$/.match("foo").inspect
- *     #=> #<MatchData "o">
- *
- *     puts /(.)(.)(.)/.match("foo").inspect
- *     #=> #<MatchData "foo" 1:"f" 2:"o" 3:"o">
- *
- *     puts /(.)(.)?(.)/.match("fo").inspect
- *     #=> #<MatchData "fo" 1:"f" 2:nil 3:"o">
- *
- */
-
-static VALUE
-match_inspect(VALUE match)
-{
-    const char *cname = rb_obj_classname(match);
-    VALUE str;
-    int i;
-    struct re_registers *regs = RMATCH(match)->regs;
-    int num_regs = regs->num_regs;
-
-    str = rb_str_buf_new2("#<");
-    rb_str_buf_cat2(str, cname);
-
-    for (i = 0; i < num_regs; i++) {
-        VALUE v;
-        rb_str_buf_cat2(str, " ");
-        if (0 < i) {
-            char buf[sizeof(i)*3+1];
-            snprintf(buf, sizeof(buf), "%d", i);
-            rb_str_buf_cat2(str, buf);
-            rb_str_buf_cat2(str, ":");
-        }
-        v = rb_reg_nth_match(i, match);
-        if (v == Qnil)
-            rb_str_buf_cat2(str, "nil");
-        else
-            rb_str_buf_append(str, rb_str_inspect(v));
-    }
-    rb_str_buf_cat2(str, ">");
-
-    return str;
-}
-
 VALUE rb_cRegexp;
 
 static void
@@ -1414,11 +1331,6 @@ rb_reg_initialize(obj, s, len, options)
 {
     struct RRegexp *re = RREGEXP(obj);
 
-    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
-	rb_raise(rb_eSecurityError, "Insecure: can't modify regexp");
-    rb_check_frozen(obj);
-    if (FL_TEST(obj, REG_LITERAL))
-	rb_raise(rb_eSecurityError, "can't modify literal regexp");
     if (re->ptr) re_free_pattern(re->ptr);
     if (re->str) free(re->str);
     re->ptr = 0;
@@ -1444,7 +1356,7 @@ rb_reg_initialize(obj, s, len, options)
     }
 
     if (options & ~0xf) {
-	rb_kcode_set_option((VALUE)re);
+	kcode_set_option((VALUE)re);
     }
     if (ruby_ignorecase) {
 	options |= RE_OPTION_IGNORECASE;
@@ -1456,9 +1368,8 @@ rb_reg_initialize(obj, s, len, options)
     re->str[len] = '\0';
     re->len = len;
     if (options & ~0xf) {
-	rb_kcode_reset_option();
+	kcode_reset_option();
     }
-    if (ruby_in_compile) FL_SET(obj, REG_LITERAL);
 }
 
 static VALUE rb_reg_s_alloc _((VALUE));
@@ -1496,7 +1407,6 @@ VALUE
 rb_reg_regcomp(str)
     VALUE str;
 {
-    volatile VALUE save_str = str;
     if (reg_cache && RREGEXP(reg_cache)->len == RSTRING(str)->len
 	&& case_cache == ruby_ignorecase
 	&& kcode_cache == reg_kcode
@@ -1505,9 +1415,8 @@ rb_reg_regcomp(str)
 
     case_cache = ruby_ignorecase;
     kcode_cache = reg_kcode;
-    reg_cache = rb_reg_new(RSTRING(str)->ptr, RSTRING(str)->len, ruby_ignorecase);
-    RB_GC_GUARD(save_str);
-    return reg_cache;
+    return reg_cache = rb_reg_new(RSTRING(str)->ptr, RSTRING(str)->len,
+				  ruby_ignorecase);
 }
 
 static int
@@ -1742,6 +1651,7 @@ rb_reg_initialize_m(argc, argv, self)
     long len;
     int flags = 0;
 
+    rb_check_frozen(self);
     if (argc == 0 || argc > 3) {
 	rb_raise(rb_eArgError, "wrong number of arguments");
     }
@@ -1835,7 +1745,7 @@ rb_reg_quote(str)
 	    goto meta_found;
 	}
     }
-    return rb_str_new3(str);
+    return str;
 
   meta_found:
     tmp = rb_str_new(0, RSTRING(str)->len*2);
@@ -1920,7 +1830,7 @@ rb_reg_s_quote(argc, argv)
     }
     StringValue(str);
     str = rb_reg_quote(str);
-    rb_kcode_reset_option();
+    kcode_reset_option();
     return str;
 }
 
@@ -1973,12 +1883,26 @@ rb_reg_options(re)
     return options;
 }
 
+
+/*
+ *  call-seq:
+ *     Regexp.union([pattern]*)   => new_str
+ *  
+ *  Return a <code>Regexp</code> object that is the union of the given
+ *  <em>pattern</em>s, i.e., will match any of its parts. The <em>pattern</em>s
+ *  can be Regexp objects, in which case their options will be preserved, or
+ *  Strings. If no arguments are given, returns <code>/(?!)/</code>.
+ *     
+ *     Regexp.union                         #=> /(?!)/
+ *     Regexp.union("penzance")             #=> /penzance/
+ *     Regexp.union("skiing", "sledding")   #=> /skiing|sledding/
+ *     Regexp.union(/dogs/, /cats/i)        #=> /(?-mix:dogs)|(?i-mx:cats)/
+ */
 static VALUE
-rb_reg_s_union(self, args0)
-    VALUE self;
-    VALUE args0;
+rb_reg_s_union(argc, argv)
+    int argc;
+    VALUE *argv;
 {
-    long argc = RARRAY_LEN(args0);
     if (argc == 0) {
         VALUE args[1];
         args[0] = rb_str_new2("(?!)");
@@ -1986,12 +1910,12 @@ rb_reg_s_union(self, args0)
     }
     else if (argc == 1) {
         VALUE v;
-        v = rb_check_convert_type(rb_ary_entry(args0, 0), T_REGEXP, "Regexp", "to_regexp");
+        v = rb_check_convert_type(argv[0], T_REGEXP, "Regexp", "to_regexp");
         if (!NIL_P(v))
             return v;
         else {
             VALUE args[1];
-            args[0] = rb_reg_s_quote(RARRAY_LEN(args0), RARRAY_PTR(args0));
+            args[0] = rb_reg_s_quote(argc, argv);
             return rb_class_new_instance(1, args, rb_cRegexp);
         }
     }
@@ -2004,7 +1928,7 @@ rb_reg_s_union(self, args0)
             volatile VALUE v;
             if (0 < i)
                 rb_str_buf_cat2(source, "|");
-            v = rb_check_convert_type(rb_ary_entry(args0, i), T_REGEXP, "Regexp", "to_regexp");
+            v = rb_check_convert_type(argv[i], T_REGEXP, "Regexp", "to_regexp");
             if (!NIL_P(v)) {
                 if (FL_TEST(v, KCODE_FIXED)) {
                     if (kcode == -1) {
@@ -2022,7 +1946,7 @@ rb_reg_s_union(self, args0)
                 v = rb_reg_to_s(v);
             }
             else {
-                args[0] = rb_ary_entry(args0, i);
+                args[0] = argv[i];
                 v = rb_reg_s_quote(1, args);
             }
             rb_str_buf_append(source, v);
@@ -2050,35 +1974,6 @@ rb_reg_s_union(self, args0)
     }
 }
 
-/*
- *  call-seq:
- *     Regexp.union(pat1, pat2, ...)            => new_regexp
- *     Regexp.union(pats_ary)                   => new_regexp
- *  
- *  Return a <code>Regexp</code> object that is the union of the given
- *  <em>pattern</em>s, i.e., will match any of its parts. The <em>pattern</em>s
- *  can be Regexp objects, in which case their options will be preserved, or
- *  Strings. If no patterns are given, returns <code>/(?!)/</code>.
- *     
- *     Regexp.union                         #=> /(?!)/
- *     Regexp.union("penzance")             #=> /penzance/
- *     Regexp.union("a+b*c")                #=> /a\+b\*c/
- *     Regexp.union("skiing", "sledding")   #=> /skiing|sledding/
- *     Regexp.union(["skiing", "sledding"]) #=> /skiing|sledding/
- *     Regexp.union(/dogs/, /cats/i)        #=> /(?-mix:dogs)|(?i-mx:cats)/
- */
-static VALUE
-rb_reg_s_union_m(VALUE self, VALUE args)
-{
-    VALUE v;
-    if (RARRAY_LEN(args) == 1 &&
-        !NIL_P(v = rb_check_array_type(rb_ary_entry(args, 0)))) {
-        return rb_reg_s_union(self, v);
-    }
-    return rb_reg_s_union(self, args);
-}
-
-/* :nodoc: */
 static VALUE
 rb_reg_init_copy(copy, re)
     VALUE copy, re;
@@ -2355,7 +2250,7 @@ Init_Regexp()
     rb_define_singleton_method(rb_cRegexp, "compile", rb_class_new_instance, -1);
     rb_define_singleton_method(rb_cRegexp, "quote", rb_reg_s_quote, -1);
     rb_define_singleton_method(rb_cRegexp, "escape", rb_reg_s_quote, -1);
-    rb_define_singleton_method(rb_cRegexp, "union", rb_reg_s_union_m, -2);
+    rb_define_singleton_method(rb_cRegexp, "union", rb_reg_s_union, -1);
     rb_define_singleton_method(rb_cRegexp, "last_match", rb_reg_s_last_match, -1);
 
     rb_define_method(rb_cRegexp, "initialize", rb_reg_initialize_m, -1);
@@ -2394,11 +2289,11 @@ Init_Regexp()
     rb_define_method(rb_cMatch, "to_a", match_to_a, 0);
     rb_define_method(rb_cMatch, "[]", match_aref, -1);
     rb_define_method(rb_cMatch, "captures", match_captures, 0);
-    rb_define_method(rb_cMatch, "values_at", match_values_at, -1);
     rb_define_method(rb_cMatch, "select", match_select, -1);
+    rb_define_method(rb_cMatch, "values_at", match_values_at, -1);
     rb_define_method(rb_cMatch, "pre_match", rb_reg_match_pre, 0);
     rb_define_method(rb_cMatch, "post_match", rb_reg_match_post, 0);
     rb_define_method(rb_cMatch, "to_s", match_to_s, 0);
-    rb_define_method(rb_cMatch, "inspect", match_inspect, 0);
+    rb_define_method(rb_cMatch, "inspect", rb_any_to_s, 0); /* in object.c */
     rb_define_method(rb_cMatch, "string", match_string, 0);
 }
