@@ -2,8 +2,8 @@
 
   signal.c -
 
-  $Author: knu $
-  $Date: 2008-06-06 19:39:57 +0900 (Fri, 06 Jun 2008) $
+  $Author: matz $
+  $Date: 2005/06/12 16:58:41 $
   created at: Tue Dec 20 10:13:44 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -21,12 +21,6 @@
 #undef SIGBUS
 #endif
 
-#if defined HAVE_SIGPROCMASK || defined HAVE_SIGSETMASK
-#define USE_TRAP_MASK 1
-#else
-#define USE_TRAP_MASK 0
-#endif
-
 #ifndef NSIG
 # ifdef DJGPP
 #  define NSIG SIGMAX
@@ -36,7 +30,7 @@
 #endif
 
 static struct signals {
-    const char *signm;
+    char *signm;
     int  signo;
 } siglist [] = {
     {"EXIT", 0},
@@ -176,7 +170,7 @@ static struct signals {
 
 static int
 signm2signo(nm)
-    const char *nm;
+    char *nm;
 {
     struct signals *sigs;
 
@@ -186,7 +180,7 @@ signm2signo(nm)
     return 0;
 }
 
-static const char*
+static char*
 signo2signm(no)
     int no;
 {
@@ -205,96 +199,6 @@ ruby_signal_name(no)
     return signo2signm(no);
 }
 
-/*
- * call-seq:
- *    SignalException.new(sig)   =>  signal_exception
- *
- *  Construct a new SignalException object.  +sig+ should be a known
- *  signal name, or a signal number.
- */
-
-static VALUE
-esignal_init(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
-{
-    int argnum = 1;
-    VALUE sig = Qnil;
-    int signo;
-    const char *signm;
-    char tmpnm[(sizeof(int)*CHAR_BIT)/3+4];
-
-    if (argc > 0) {
-	sig = argv[0];
-	if (FIXNUM_P(sig)) argnum = 2;
-    }
-    if (argc < 1 || argnum < argc) {
-	rb_raise(rb_eArgError, "wrong number of arguments (%d for %d)",
-		 argc, argnum);
-    }
-    if (argnum == 2) {
-	signo = FIX2INT(sig);
-	if (signo < 0 || signo > NSIG) {
-	    rb_raise(rb_eArgError, "invalid signal number (%d)", signo);
-	}
-	if (argc > 1) {
-	    sig = argv[1];
-	}
-	else {
-	    signm = signo2signm(signo);
-	    if (signm) {
-		snprintf(tmpnm, sizeof(tmpnm), "SIG%s", signm);
-	    }
-	    else {
-		snprintf(tmpnm, sizeof(tmpnm), "SIG%u", signo);
-	    }
-	    sig = rb_str_new2(signm = tmpnm);
-	}
-    }
-    else {
-	signm = SYMBOL_P(sig) ? rb_id2name(SYM2ID(sig)) : StringValuePtr(sig);
-	if (strncmp(signm, "SIG", 3) == 0) signm += 3;
-	signo = signm2signo(signm);
-	if (!signo) {
-	    rb_raise(rb_eArgError, "unsupported name `SIG%s'", signm);
-	}
-	if (SYMBOL_P(sig)) {
-	    sig = rb_str_new2(signm);
-	}
-    }
-    rb_call_super(1, &sig);
-    rb_iv_set(self, "signo", INT2NUM(signo));
-
-    return self;
-}
-
-static VALUE
-interrupt_init(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
-{
-    VALUE args[2];
-
-    args[0] = INT2FIX(SIGINT);
-    rb_scan_args(argc, argv, "01", &args[1]);
-
-    return rb_call_super(2, args);
-}
-
-void
-ruby_default_signal(sig)
-    int sig;
-{
-#ifndef MACOS_UNUSE_SIGNAL
-    extern rb_pid_t getpid _((void));
-
-    signal(sig, SIG_DFL);
-    kill(getpid(), sig);
-#endif
-}
-
 /*
  *  call-seq:
  *     Process.kill(signal, pid, ...)    => fixnum
@@ -327,7 +231,7 @@ rb_f_kill(argc, argv)
     int negative = 0;
     int sig;
     int i;
-    const char *s;
+    char *s;
 
     rb_secure(2);
     if (argc < 2)
@@ -400,7 +304,6 @@ static struct {
     int safe;
 } trap_list[NSIG];
 static rb_atomic_t trap_pending_list[NSIG];
-static char rb_trap_accept_nativethreads[NSIG];
 rb_atomic_t rb_trap_pending;
 rb_atomic_t rb_trap_immediate;
 int rb_prohibit_interrupt = 1;
@@ -432,15 +335,21 @@ ruby_signal(signum, handler)
 {
     struct sigaction sigact, old;
 
-    rb_trap_accept_nativethreads[signum] = 0;
-
     sigact.sa_handler = handler;
     sigemptyset(&sigact.sa_mask);
     sigact.sa_flags = 0;
-# ifdef SA_NOCLDWAIT
+#if defined(SA_RESTART)
+    /* All other signals but VTALRM shall restart restartable syscall
+       VTALRM will cause EINTR to syscall if interrupted.
+    */
+    if (signum != SIGVTALRM) {
+        sigact.sa_flags |= SA_RESTART; /* SVR4, 4.3+BSD */
+    }
+#endif
+#ifdef SA_NOCLDWAIT
     if (signum == SIGCHLD && handler == SIG_IGN)
 	sigact.sa_flags |= SA_NOCLDWAIT;
-# endif
+#endif
     sigaction(signum, &sigact, &old);
     return old.sa_handler;
 }
@@ -452,46 +361,9 @@ posix_signal(signum, handler)
 {
     ruby_signal(signum, handler);
 }
-
-# ifdef HAVE_NATIVETHREAD
-static sighandler_t
-ruby_nativethread_signal(signum, handler)
-    int signum;
-    sighandler_t handler;
-{
-    sighandler_t old;
-
-    old = ruby_signal(signum, handler);
-    rb_trap_accept_nativethreads[signum] = 1;
-    return old;
-}
-
-void
-posix_nativethread_signal(signum, handler)
-    int signum;
-    sighandler_t handler;
-{
-    ruby_nativethread_signal(signum, handler);
-}
-# endif
-
-#else /* !POSIX_SIGNAL */
-#define ruby_signal(sig,handler) (rb_trap_accept_nativethreads[sig] = 0, signal((sig),(handler)))
-
-# ifdef HAVE_NATIVETHREAD
-static sighandler_t
-ruby_nativethread_signal(signum, handler)
-    int signum;
-    sighandler_t handler;
-{
-    sighandler_t old;
-
-    old = signal(signum, handler);
-    rb_trap_accept_nativethreads[signum] = 1;
-    return old;
-}
-# endif
-#endif /* POSIX_SIGNAL */
+#else
+#define ruby_signal(sig,handler) signal((sig),(handler))
+#endif
 
 static void signal_exec _((int sig));
 static void
@@ -509,9 +381,6 @@ signal_exec(sig)
 #ifdef SIGQUIT
 	  case SIGQUIT:
 #endif
-#ifdef SIGTERM
-	  case SIGTERM:
-#endif
 #ifdef SIGALRM
 	  case SIGALRM:
 #endif
@@ -521,40 +390,15 @@ signal_exec(sig)
 #ifdef SIGUSR2
 	  case SIGUSR2:
 #endif
-	    rb_thread_signal_raise(sig);
+	    rb_thread_signal_raise(signo2signm(sig));
 	    break;
 	}
     }
-    else if (trap_list[sig].cmd == Qundef) {
-	rb_thread_signal_exit();
-    }
     else {
 	rb_thread_trap_eval(trap_list[sig].cmd, sig, trap_list[sig].safe);
     }
 }
 
-#if defined(HAVE_NATIVETHREAD) && defined(HAVE_NATIVETHREAD_KILL)
-static void
-sigsend_to_ruby_thread(int sig)
-{
-# ifdef HAVE_SIGPROCMASK
-    sigset_t mask, old_mask;
-# else
-    int mask, old_mask;
-# endif
-
-# ifdef HAVE_SIGPROCMASK
-    sigfillset(&mask);
-    sigprocmask(SIG_BLOCK, &mask, &old_mask);
-# else
-    mask = sigblock(~0);
-    sigsetmask(mask);
-# endif
-
-    ruby_native_thread_kill(sig);
-}
-#endif
-
 static RETSIGTYPE sighandler _((int));
 static RETSIGTYPE
 sighandler(sig)
@@ -570,32 +414,17 @@ sighandler(sig)
 	rb_bug("trap_handler: Bad signal %d", sig);
     }
 
-#if defined(HAVE_NATIVETHREAD) && defined(HAVE_NATIVETHREAD_KILL)
-    if (!is_ruby_native_thread() && !rb_trap_accept_nativethreads[sig]) {
-	sigsend_to_ruby_thread(sig);
-	return;
-    }
-#endif
-
 #if !defined(BSD_SIGNAL) && !defined(POSIX_SIGNAL)
-    if (rb_trap_accept_nativethreads[sig]) {
-	ruby_nativethread_signal(sig, sighandler);
-    }
-    else {
-	ruby_signal(sig, sighandler);
-    }
+    ruby_signal(sig, sighandler);
 #endif
 
-    if (trap_list[sig].cmd == 0 && ATOMIC_TEST(rb_trap_immediate)) {
+    if (ATOMIC_TEST(rb_trap_immediate)) {
 	IN_MAIN_CONTEXT(signal_exec, sig);
 	ATOMIC_SET(rb_trap_immediate, 1);
     }
     else {
 	ATOMIC_INC(rb_trap_pending);
 	ATOMIC_INC(trap_pending_list[sig]);
-#ifdef _WIN32
-	rb_w32_interrupted();
-#endif
     }
 }
 
@@ -605,13 +434,6 @@ static RETSIGTYPE
 sigbus(sig)
     int sig;
 {
-#if defined(HAVE_NATIVETHREAD) && defined(HAVE_NATIVETHREAD_KILL)
-    if (!is_ruby_native_thread() && !rb_trap_accept_nativethreads[sig]) {
-        sigsend_to_ruby_thread(sig);
-        return;
-    }
-#endif
-
     rb_bug("Bus Error");
 }
 #endif
@@ -622,15 +444,6 @@ static RETSIGTYPE
 sigsegv(sig)
     int sig;
 {
-#if defined(HAVE_NATIVETHREAD) && defined(HAVE_NATIVETHREAD_KILL)
-    if (!is_ruby_native_thread() && !rb_trap_accept_nativethreads[sig]) {
-        sigsend_to_ruby_thread(sig);
-        return;
-    }
-#endif
-
-    extern int ruby_gc_stress;
-    ruby_gc_stress = 0;
     rb_bug("Segmentation fault");
 }
 #endif
@@ -675,7 +488,7 @@ rb_trap_exec()
 }
 
 struct trap_arg {
-#if USE_TRAP_MASK
+#ifndef _WIN32
 # ifdef HAVE_SIGPROCMASK
     sigset_t mask;
 # else
@@ -685,13 +498,11 @@ struct trap_arg {
     VALUE sig, cmd;
 };
 
-#if USE_TRAP_MASK
 # ifdef HAVE_SIGPROCMASK
 static sigset_t trap_last_mask;
 # else
 static int trap_last_mask;
 # endif
-#endif
 
 static RETSIGTYPE sigexit _((int));
 static RETSIGTYPE
@@ -708,7 +519,7 @@ trap(arg)
     sighandler_t func, oldfunc;
     VALUE command, oldcmd;
     int sig = -1;
-    const char *s;
+    char *s;
 
     func = sighandler;
     command = arg->cmd;
@@ -767,7 +578,7 @@ trap(arg)
 	    rb_raise(rb_eArgError, "unsupported signal SIG%s", s);
     }
 
-    if (sig < 0 || sig >= NSIG) {
+    if (sig < 0 || sig > NSIG) {
 	rb_raise(rb_eArgError, "invalid signal number (%d)", sig);
     }
 #if defined(HAVE_SETITIMER)
@@ -784,9 +595,6 @@ trap(arg)
 #ifdef SIGQUIT
 	  case SIGQUIT:
 #endif
-#ifdef SIGTERM
-	  case SIGTERM:
-#endif
 #ifdef SIGALRM
 	  case SIGALRM:
 #endif
@@ -826,7 +634,7 @@ trap(arg)
     trap_list[sig].cmd = command;
     trap_list[sig].safe = ruby_safe_level;
     /* enable at least specified signal. */
-#if USE_TRAP_MASK
+#ifndef _WIN32
 #ifdef HAVE_SIGPROCMASK
     sigdelset(&arg->mask, sig);
 #else
@@ -836,7 +644,7 @@ trap(arg)
     return oldcmd;
 }
 
-#if USE_TRAP_MASK
+#ifndef _WIN32
 static VALUE
 trap_ensure(arg)
     struct trap_arg *arg;
@@ -855,7 +663,7 @@ trap_ensure(arg)
 void
 rb_trap_restore_mask()
 {
-#if USE_TRAP_MASK
+#ifndef _WIN32
 # ifdef HAVE_SIGPROCMASK
     sigprocmask(SIG_SETMASK, &trap_last_mask, NULL);
 # else
@@ -915,7 +723,7 @@ sig_trap(argc, argv)
     if (OBJ_TAINTED(arg.cmd)) {
 	rb_raise(rb_eSecurityError, "Insecure: tainted signal trap");
     }
-#if USE_TRAP_MASK
+#ifndef _WIN32
     /* disable interrupt */
 # ifdef HAVE_SIGPROCMASK
     sigfillset(&arg.mask);
@@ -964,42 +772,12 @@ install_sighandler(signum, handler)
     }
 }
 
-#if 0
-/* 
- *   If you write a handler which works on any native thread
- *   (even if the thread is NOT a ruby's one), please enable
- *   this function and use it to install the handler, instead
- *   of `install_sighandler()'.
- */
-#ifdef HAVE_NATIVETHREAD
-static void
-install_nativethread_sighandler(signum, handler)
-    int signum;
-    sighandler_t handler;
-{
-    sighandler_t old;
-    int old_st;
-
-    old_st = rb_trap_accept_nativethreads[signum];
-    old = ruby_nativethread_signal(signum, handler);
-    if (old != SIG_DFL) {
-        if (old_st) {
-            ruby_nativethread_signal(signum, old);
-        } else {
-            ruby_signal(signum, old);
-        }
-    }
-}
-#endif
-#endif
-
-#if defined(SIGCLD) || defined(SIGCHLD)
 static void
 init_sigchld(sig)
     int sig;
 {
     sighandler_t oldfunc;
-#if USE_TRAP_MASK
+#ifndef _WIN32
 # ifdef HAVE_SIGPROCMASK
     sigset_t mask;
 # else
@@ -1007,7 +785,7 @@ init_sigchld(sig)
 # endif
 #endif
 
-#if USE_TRAP_MASK
+#ifndef _WIN32
     /* disable interrupt */
 # ifdef HAVE_SIGPROCMASK
     sigfillset(&mask);
@@ -1024,7 +802,7 @@ init_sigchld(sig)
 	trap_list[sig].cmd = 0;
     }
 
-#if USE_TRAP_MASK
+#ifndef _WIN32
 #ifdef HAVE_SIGPROCMASK
     sigdelset(&mask, sig);
     sigprocmask(SIG_SETMASK, &mask, NULL);
@@ -1035,7 +813,6 @@ init_sigchld(sig)
     trap_last_mask = mask;
 #endif
 }
-#endif
 
 /*
  * Many operating systems allow signals to be sent to running
@@ -1084,11 +861,6 @@ Init_signal()
     rb_define_module_function(mSignal, "trap", sig_trap, -1);
     rb_define_module_function(mSignal, "list", sig_list, 0);
 
-    rb_define_method(rb_eSignal, "initialize", esignal_init, -1);
-    rb_attr(rb_eSignal, rb_intern("signo"), 1, 0, 0);
-    rb_alias(rb_eSignal, rb_intern("signm"), rb_intern("message"));
-    rb_define_method(rb_eInterrupt, "initialize", interrupt_init, -1);
-
     install_sighandler(SIGINT, sighandler);
 #ifdef SIGHUP
     install_sighandler(SIGHUP, sighandler);
@@ -1096,9 +868,6 @@ Init_signal()
 #ifdef SIGQUIT
     install_sighandler(SIGQUIT, sighandler);
 #endif
-#ifdef SIGTERM
-    install_sighandler(SIGTERM, sighandler);
-#endif
 #ifdef SIGALRM
     install_sighandler(SIGALRM, sighandler);
 #endif
@@ -1119,9 +888,10 @@ Init_signal()
     install_sighandler(SIGPIPE, sigpipe);
 #endif
 
-#if defined(SIGCLD)
+#ifdef SIGCLD
     init_sigchld(SIGCLD);
-#elif defined(SIGCHLD)
+#endif
+#ifdef SIGCHLD
     init_sigchld(SIGCHLD);
 #endif
 
