@@ -3,7 +3,7 @@
   sprintf.c -
 
   $Author: matz $
-  $Date: 2004/12/08 06:22:51 $
+  $Date: 2005/09/16 03:04:59 $
   created at: Fri Oct 15 10:39:26 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -107,7 +107,7 @@ sign_bits(base, p)
 	       (posarg = -1, GETNTHARG(n))))
 
 #define GETNTHARG(nth) \
-    ((nth >= argc) ? (rb_raise(rb_eArgError, "too few arguments."), 0) : argv[nth])
+    ((nth >= argc) ? (rb_raise(rb_eArgError, "too few arguments"), 0) : argv[nth])
 
 #define GETASTER(val) do { \
     t = p++; \
@@ -146,7 +146,7 @@ sign_bits(base, p)
  *
  *    Flag     | Applies to   | Meaning
  *    ---------+--------------+-----------------------------------------
- *    space    | bdeEfgGioxXu | Leave a space at the start of 
+ *    space    | bdeEfgGiouxX | Leave a space at the start of 
  *             |              | positive numbers.
  *    ---------+--------------+-----------------------------------------
  *    (digit)$ | all          | Specifies the absolute argument number
@@ -162,11 +162,11 @@ sign_bits(base, p)
  *             |              | point to be added, even if no digits follow.
  *             |              | For `g' and 'G', do not remove trailing zeros.
  *    ---------+--------------+-----------------------------------------
- *    +        | bdeEfgGioxXu | Add a leading plus sign to positive numbers.
+ *    +        | bdeEfgGiouxX | Add a leading plus sign to positive numbers.
  *    ---------+--------------+-----------------------------------------
  *    -        | all          | Left-justify the result of this conversion.
  *    ---------+--------------+-----------------------------------------
- *    0 (zero) | all          | Pad with zeros, not spaces.
+ *    0 (zero) | bdeEfgGiouxX | Pad with zeros, not spaces.
  *    ---------+--------------+-----------------------------------------
  *    *        | all          | Use the next argument as the field width. 
  *             |              | If negative, left-justify the result. If the
@@ -771,11 +771,12 @@ rb_f_sprintf(argc, argv)
     }
 
   sprint_exit:
-    /* XXX - We cannot validiate the number of arguments because
-     *       the format string may contain `n$'-style argument selector.
+    /* XXX - We cannot validiate the number of arguments if (digit)$ style used.
      */
-    if (RTEST(ruby_verbose) && posarg >= 0 && nextarg < argc) {
-	rb_raise(rb_eArgError, "too many arguments for format string");
+    if (posarg >= 0 && nextarg < argc) {
+	const char *mesg = "too many arguments for format string";
+	if (RTEST(ruby_debug)) rb_raise(rb_eArgError, mesg);
+	if (RTEST(ruby_verbose)) rb_warn(mesg);
     }
     rb_str_resize(result, blen);
 
