@@ -2,8 +2,8 @@
 
   variable.c -
 
-  $Author: shyouhei $
-  $Date: 2011-06-02 19:02:20 +0900 (Thu, 02 Jun 2011) $
+  $Author: matz $
+  $Date: 2005/07/27 03:07:01 $
   created at: Tue Apr 19 23:55:15 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -18,7 +18,7 @@
 #include "st.h"
 #include "util.h"
 
-st_table *rb_global_tbl;
+static st_table *rb_global_tbl;
 st_table *rb_class_tbl;
 static ID autoload, classpath, tmp_classpath;
 
@@ -195,7 +195,7 @@ rb_class_path(klass)
 	return path;
     }
     else {
-	const char *s = "Class";
+	char *s = "Class";
 	size_t len;
 
 	if (TYPE(klass) == T_MODULE) {
@@ -241,21 +241,16 @@ rb_path2class(path)
     const char *pbeg, *p;
     ID id;
     VALUE c = rb_cObject;
-    VALUE str = 0;
 
     if (path[0] == '#') {
 	rb_raise(rb_eArgError, "can't retrieve anonymous class %s", path);
     }
     pbeg = p = path;
     while (*p) {
+	VALUE str;
+
 	while (*p && *p != ':') p++;
-	if (str) {
-	    RSTRING(str)->len = 0;
-	    rb_str_cat(str, pbeg, p-pbeg);
-	}
-	else {
-	    str = rb_str_new(pbeg, p-pbeg);
-	}
+	str = rb_str_new(pbeg, p-pbeg);
 	id = rb_intern(RSTRING(str)->ptr);
 	if (p[0] == ':') {
 	    if (p[1] != ':') goto undefined_class;
@@ -294,14 +289,14 @@ rb_class_name(klass)
     return rb_class_path(rb_class_real(klass));
 }
 
-const char *
+char *
 rb_class2name(klass)
     VALUE klass;
 {
     return RSTRING(rb_class_name(klass))->ptr;
 }
 
-const char *
+char *
 rb_obj_classname(obj)
     VALUE obj;
 {
@@ -347,9 +342,8 @@ rb_global_entry(id)
     ID id;
 {
     struct global_entry *entry;
-    st_data_t data;
 
-    if (!st_lookup(rb_global_tbl, id, &data)) {
+    if (!st_lookup(rb_global_tbl, id, (st_data_t *)&entry)) {
 	struct global_variable *var;
 	entry = ALLOC(struct global_entry);
 	var = ALLOC(struct global_variable);
@@ -365,9 +359,6 @@ rb_global_entry(id)
 	var->trace = 0;
 	st_add_direct(rb_global_tbl, id, (st_data_t)entry);
     }
-    else {
-	entry = (struct global_entry *)data;
-    }
     return entry;
 }
 
@@ -478,9 +469,7 @@ mark_global_entry(key, entry)
 void
 rb_gc_mark_global_tbl()
 {
-    if (rb_global_tbl) {
-	st_foreach(rb_global_tbl, mark_global_entry, 0);
-    }
+    st_foreach(rb_global_tbl, mark_global_entry, 0);
 }
 
 static ID
@@ -645,16 +634,14 @@ rb_f_untrace_var(argc, argv)
     ID id;
     struct global_entry *entry;
     struct trace_var *trace;
-    st_data_t data;
 
-    rb_secure(4);
     rb_scan_args(argc, argv, "11", &var, &cmd);
     id = rb_to_id(var);
-    if (!st_lookup(rb_global_tbl, id, &data)) {
+    if (!st_lookup(rb_global_tbl, id, (st_data_t *)&entry)) {
 	rb_name_error(id, "undefined global variable %s", rb_id2name(id));
     }
 
-    trace = (entry = (struct global_entry *)data)->var->trace;
+    trace = entry->var->trace;
     if (NIL_P(cmd)) {
 	VALUE ary = rb_ary_new();
 
@@ -790,7 +777,7 @@ rb_f_global_variables()
 {
     VALUE ary = rb_ary_new();
     char buf[4];
-    const char *s = "&`'+123456789";
+    char *s = "&`'+123456789";
 
     st_foreach(rb_global_tbl, gvar_i, ary);
     if (!NIL_P(rb_backref_get())) {
@@ -808,18 +795,17 @@ rb_alias_variable(name1, name2)
     ID name2;
 {
     struct global_entry *entry1, *entry2;
-    st_data_t data1;
 
     if (rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't alias global variable");
 
     entry2 = rb_global_entry(name2);
-    if (!st_lookup(rb_global_tbl, name1, &data1)) {
+    if (!st_lookup(rb_global_tbl, name1, (st_data_t *)&entry1)) {
 	entry1 = ALLOC(struct global_entry);
 	entry1->id = name1;
 	st_add_direct(rb_global_tbl, name1, (st_data_t)entry1);
     }
-    else if ((entry1 = (struct global_entry *)data1)->var != entry2->var) {
+    else if (entry1->var != entry2->var) {
 	struct global_variable *var = entry1->var;
 	if (var->block_trace) {
 	    rb_raise(rb_eRuntimeError, "can't alias in tracer");
@@ -849,12 +835,12 @@ st_table*
 rb_generic_ivar_table(obj)
     VALUE obj;
 {
-    st_data_t tbl;
+    st_table *tbl;
 
     if (!FL_TEST(obj, FL_EXIVAR)) return 0;
     if (!generic_iv_tbl) return 0;
-    if (!st_lookup(generic_iv_tbl, obj, &tbl)) return 0;
-    return (st_table *)tbl;
+    if (!st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) return 0;
+    return tbl;
 }
 
 static VALUE
@@ -863,15 +849,15 @@ generic_ivar_get(obj, id, warn)
     ID id;
     int warn;
 {
-    st_data_t tbl;
+    st_table *tbl;
     VALUE val;
 
     if (generic_iv_tbl) {
-	if (st_lookup(generic_iv_tbl, obj, &tbl)) {
-	    if (st_lookup((st_table *)tbl, id, &val)) {
-		return val;
-	    }
+      if (st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) {
+	if (st_lookup(tbl, id, &val)) {
+	  return val;
 	}
+      }
     }
     if (warn) {
 	rb_warning("instance variable %s not initialized", rb_id2name(id));
@@ -886,7 +872,6 @@ generic_ivar_set(obj, id, val)
     VALUE val;
 {
     st_table *tbl;
-    st_data_t data;
 
     if (rb_special_const_p(obj)) {
 	special_generic_ivar = 1;
@@ -895,14 +880,14 @@ generic_ivar_set(obj, id, val)
 	generic_iv_tbl = st_init_numtable();
     }
 
-    if (!st_lookup(generic_iv_tbl, obj, &data)) {
+    if (!st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) {
 	FL_SET(obj, FL_EXIVAR);
 	tbl = st_init_numtable();
 	st_add_direct(generic_iv_tbl, obj, (st_data_t)tbl);
 	st_add_direct(tbl, id, val);
 	return;
     }
-    st_insert((st_table *)data, id, val);
+    st_insert(tbl, id, val);
 }
 
 static VALUE
@@ -911,12 +896,10 @@ generic_ivar_defined(obj, id)
     ID id;
 {
     st_table *tbl;
-    st_data_t data;
     VALUE val;
 
     if (!generic_iv_tbl) return Qfalse;
-    if (!st_lookup(generic_iv_tbl, obj, &data)) return Qfalse;
-    tbl = (st_table *)data;
+    if (!st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) return Qfalse;
     if (st_lookup(tbl, id, &val)) {
 	return Qtrue;
     }
@@ -930,16 +913,14 @@ generic_ivar_remove(obj, id, valp)
     VALUE *valp;
 {
     st_table *tbl;
-    st_data_t data;
     int status;
 
     if (!generic_iv_tbl) return 0;
-    if (!st_lookup(generic_iv_tbl, obj, &data)) return 0;
-    tbl = (st_table *)data;
+    if (!st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) return 0;
     status = st_delete(tbl, &id, valp);
     if (tbl->num_entries == 0) {
-	st_delete(generic_iv_tbl, &obj, &data);
-	st_free_table((st_table *)data);
+	st_delete(generic_iv_tbl, &obj, (st_data_t *)&tbl);
+	st_free_table(tbl);
     }
     return status;
 }
@@ -948,11 +929,11 @@ void
 rb_mark_generic_ivar(obj)
     VALUE obj;
 {
-    st_data_t tbl;
+    st_table *tbl;
 
     if (!generic_iv_tbl) return;
-    if (st_lookup(generic_iv_tbl, obj, &tbl)) {
-	rb_mark_tbl((st_table *)tbl);
+    if (st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) {
+	rb_mark_tbl(tbl);
     }
 }
 
@@ -988,26 +969,24 @@ void
 rb_free_generic_ivar(obj)
     VALUE obj;
 {
-    st_data_t tbl;
+    st_table *tbl;
 
-    if (!generic_iv_tbl) return;
-    if (st_delete(generic_iv_tbl, &obj, &tbl))
-	st_free_table((st_table *)tbl);
+    if (st_delete(generic_iv_tbl, &obj, (st_data_t *)&tbl))
+	st_free_table(tbl);
 }
 
 void
 rb_copy_generic_ivar(clone, obj)
     VALUE clone, obj;
 {
-    st_data_t data;
+    st_table *tbl;
 
     if (!generic_iv_tbl) return;
-    if (!FL_TEST(obj, FL_EXIVAR)) return;
-    if (st_lookup(generic_iv_tbl, obj, &data)) {
-	st_table *tbl = (st_table *)data;
+    if (st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) {
+	st_table *old;
 
-	if (st_lookup(generic_iv_tbl, clone, &data)) {
-	    st_free_table((st_table *)data);
+	if (st_lookup(generic_iv_tbl, clone, (st_data_t *)&old)) {
+	    st_free_table(old);
 	    st_insert(generic_iv_tbl, clone, (st_data_t)st_copy(tbl));
 	}
 	else {
@@ -1148,10 +1127,10 @@ rb_obj_instance_variables(obj)
       default:
 	if (!generic_iv_tbl) break;
 	if (FL_TEST(obj, FL_EXIVAR) || rb_special_const_p(obj)) {
-	    st_data_t tbl;
+	    st_table *tbl;
 
-	    if (st_lookup(generic_iv_tbl, obj, &tbl)) {
-		st_foreach_safe((st_table *)tbl, ivar_i, ary);
+	    if (st_lookup(generic_iv_tbl, obj, (st_data_t *)&tbl)) {
+		st_foreach_safe(tbl, ivar_i, ary);
 	    }
 	}
 	break;
@@ -1331,69 +1310,37 @@ autoload_delete(mod, id)
     VALUE val;
     st_data_t load = 0;
 
+    st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, 0);
     if (st_lookup(RCLASS(mod)->iv_tbl, autoload, &val)) {
 	struct st_table *tbl = check_autoload_table(val);
 
 	st_delete(tbl, (st_data_t*)&id, &load);
 
 	if (tbl->num_entries == 0) {
+	    DATA_PTR(val) = 0;
+	    st_free_table(tbl);
 	    id = autoload;
-	    st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val);
+	    if (st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val)) {
+		rb_gc_force_recycle(val);
+	    }
 	}
     }
 
     return (NODE *)load;
 }
 
-static NODE *
-autoload_node(mod, id)
-    VALUE mod;
-    ID id;
-{
-    st_data_t val, load, n = id;
-    struct st_table *p, *q;
-
-    if ((p = RCLASS(mod)->iv_tbl) == 0) {
-        return 0;
-    }
-    else if (!st_lookup(p, n, &val)) {
-        return 0;
-    }
-    else if (val != Qundef) {
-        return 0;
-    }
-    else if (!st_lookup(p, autoload, &val)) {
-        return 0;
-    }
-    else if ((q = check_autoload_table((VALUE)val)) == 0) {
-        return 0;
-    }
-    else if (!st_lookup(q, n, &load)) {
-        return 0;
-    }
-    else {
-        return (NODE *)load;
-    }
-}
-
-VALUE
+void
 rb_autoload_load(klass, id)
     VALUE klass;
     ID id;
 {
-    NODE *load = 0;
-    VALUE ret = 0;
+    VALUE file;
+    NODE *load = autoload_delete(klass, id);
 
-    if ((load = autoload_node(klass, id)) == 0) {
-        return 0;
-    }
-    else if ((ret = rb_require_safe(load->nd_lit, load->nd_nth)) == Qfalse) {
-        return 0;
-    }
-    else {
-        (void) autoload_delete(klass, id);
-        return ret;
+    if (!load || !(file = load->nd_lit) || rb_provided(RSTRING(file)->ptr)) {
+	const_missing(klass, id);
     }
+    rb_require_safe(file, load->nd_nth);
 }
 
 static VALUE
@@ -1421,8 +1368,12 @@ autoload_file(mod, id)
     /* already loaded but not defined */
     st_delete(tbl, (st_data_t*)&id, 0);
     if (!tbl->num_entries) {
+	DATA_PTR(val) = 0;
+	st_free_table(tbl);
 	id = autoload;
-	st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val);
+	if (st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val)) {
+	    rb_gc_force_recycle(val);
+	}
     }
     return Qnil;
 }
@@ -1456,14 +1407,7 @@ rb_const_get_0(klass, id, exclude, recurse)
 	while (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,&value)) {
 	    if (value == Qundef) {
 		rb_autoload_load(tmp, id);
-                st_lookup(RCLASS(tmp)->iv_tbl, id, &value);
-                if (value == Qundef) {
-                    /* the autoload above did not assign a constant */
-                    break;
-                }
-                else {
-                    continue;
-                }
+		continue;
 	    }
 	    if (exclude && tmp == rb_cObject && klass != rb_cObject) {
 		rb_warn("toplevel constant %s referenced by %s::%s",
@@ -1689,7 +1633,7 @@ mod_av_set(klass, id, val, isconst)
     VALUE val;
     int isconst;
 {
-    const char *dest = isconst ? "constant" : "class variable";
+    char *dest = isconst ? "constant" : "class variable";
 
     if (!OBJ_TAINTED(klass) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't set %s", dest);
@@ -1724,10 +1668,6 @@ rb_const_set(klass, id, val)
     ID id;
     VALUE val;
 {
-    if (NIL_P(klass)) {
-	rb_raise(rb_eTypeError, "no class/module to define constant %s",
-		 rb_id2name(id));
-    }
     mod_av_set(klass, id, val, Qtrue);
 }
 
@@ -1740,7 +1680,7 @@ rb_define_const(klass, name, val)
     ID id = rb_intern(name);
 
     if (!rb_is_const_id(id)) {
-	rb_warn("rb_define_const: invalid name `%s' for constant", name);
+	rb_warn("rb_define_const: invalide name `%s' for constant", name);
     }
     if (klass == rb_cObject) {
 	rb_secure(4);
